///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////



import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.text.Document;
import javax.swing.text.BadLocationException;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;



//Class that will build the advanced configuration window
public class Advanced_software_window
    extends JDialog implements ActionListener{


    //Main window
    private Main_window main_window = null;

    //Main configuration window
    private  Configuration_software_main_window main_conf_window = null;

    //Software configuration window
    private Configuration_software_window soft_window = null;

    //Main panel
    private JPanel main_conf_panel = null;

    //Configuration text panel
    private JTextPane text_panel = null;

    //Scroll panel
    private JScrollPane scroll_panel = null;

    //Buttons
    private JButton save = null;
    private JButton reset = null;
    private JButton cancel = null;

    //Layout and constraints
    private GridBagLayout gb_layout = null;
    private GridBagConstraints constraints = null;

    //Dimensions
    private Dimension button_dimension = new Dimension(150,30);
    private Dimension text_dimension = new Dimension(600,600);

    //Reader and buffer
    private FileReader reader = null;
    private BufferedReader buffer_reader = null;  

    //Writer and buffer
    private FileWriter writer = null;
    private BufferedWriter buffer_writer = null;  

 
   //Current configuration file
    private String current_conf_file;

    //Current default configuration file
    private String default_conf_file;

    //Current new configuration file
    private String new_conf_file;

    //Content of the configuration file
    private String content = "";

    //Constructor
    public Advanced_software_window(Main_window main_window,
				    Configuration_software_main_window main_conf_window,
				    Configuration_software_window soft_window,
				    String current_conf_file,
				    String default_conf_file,
				    String new_conf_file){

	super(main_conf_window,true);
	this.main_window = main_window;
	this.main_conf_window = main_conf_window;
	this.setTitle(soft_window.getTitle());
	this.setLocationRelativeTo(main_conf_window);
	this.setResizable(false);
	this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	this.soft_window = soft_window;

	this.current_conf_file = current_conf_file;
	this.default_conf_file = default_conf_file;
	this.new_conf_file = new_conf_file;

    }

    //Build the window
    public void build(){

	//Layout and constraints definitions
	gb_layout = new GridBagLayout();
	constraints = new GridBagConstraints();
	constraints.weightx = 1;
 	constraints.weighty = 0.1;
	constraints.insets = new Insets(10,10,10,10);

	main_conf_panel = new JPanel(gb_layout);
	main_conf_panel.setBackground(Color.white);

	//Text and scroll panels
	text_panel = new JTextPane();
	text_panel.setEditable(true);
	text_panel.setPreferredSize(text_dimension);
	text_panel.setMinimumSize(text_dimension);
	scroll_panel = new JScrollPane(text_panel);


	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 3;
	constraints.gridheight = 1;
	constraints.fill = GridBagConstraints.BOTH;
	main_conf_panel.add(scroll_panel,constraints);

	//Buttons
	save = new JButton("Save");
	save.addActionListener(this);
	save.setPreferredSize(button_dimension);
	save.setMinimumSize(button_dimension);
	constraints.gridx = 0;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	constraints.weightx = 0;
 	constraints.weighty = 0;
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.CENTER;
	main_conf_panel.add(save,constraints);

	reset = new JButton("Reset to default");
	reset.addActionListener(this);
	reset.setPreferredSize(button_dimension);
	reset.setMinimumSize(button_dimension);
	constraints.gridx = 1;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	constraints.weightx = 0.1;
 	constraints.weighty = 0;
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.CENTER;
	main_conf_panel.add(reset,constraints);

	cancel = new JButton("Cancel");
	cancel.addActionListener(this);
	cancel.setPreferredSize(button_dimension);
	cancel.setMinimumSize(button_dimension);
	constraints.gridx = 2;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	constraints.weightx = 0;
	constraints.weighty = 0;
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.CENTER;
	main_conf_panel.add(cancel,constraints);


	//Open the configuration file and print it in the text panel
	try{
	    reader = new FileReader(current_conf_file);
	    buffer_reader = new BufferedReader(reader);
	    
	    String line = "";
	    while((line = buffer_reader.readLine()) != null){
		content += line+"\n";
	    }
	}
	catch(IOException ioe){
	    ioe.printStackTrace();
	}
	finally{
	    try{
		buffer_reader.close();
		reader.close();
	    }
	    catch(IOException ioe_close){
		ioe_close.printStackTrace();
	    }
	}
	
	Document doc = text_panel.getDocument();
	try{
	    doc.insertString(0,
			     content,
			     null);
	    text_panel.setCaretPosition(0);
	}
	catch(BadLocationException ble){
	    ble.printStackTrace();
	}


	getContentPane().add(main_conf_panel);
	pack();
	setVisible(true);

    }



    public void actionPerformed(ActionEvent e){


	//Cancel button
	if(e.getSource() == cancel){

	    soft_window.set_new_conf_file(current_conf_file);
	    setVisible(false);

	}

	else{

	    //Save button
	    if(e.getSource() == save){

		try{
		    writer = new FileWriter(new_conf_file);
		    buffer_writer = new BufferedWriter(writer);
		
		    Document doc = text_panel.getDocument();
		    try{
			content = doc.getText(0,doc.getLength());
		    }
		    catch(BadLocationException ble){
			ble.printStackTrace();
		    }


		    buffer_writer.write(content);
		}
		catch(IOException ioe){
		    ioe.printStackTrace();
		}
		finally{
		    try{
			buffer_writer.flush();
			buffer_writer.close();
			writer.close();
		    }
		    catch(IOException ioe_close){
			ioe_close.printStackTrace();
		    }
		}
		soft_window.set_new_conf_file(new_conf_file);
		setVisible(false);
	    }

	    //reset button
	    else{
		
		String default_content = "";

		//Open the default configuration file and print it 
		//in the text panel
		try{
		    reader = new FileReader(default_conf_file);
		    buffer_reader = new BufferedReader(reader);
		    
		    String line = "";
		    while((line = buffer_reader.readLine()) != null){
			default_content += line+"\n";
		    }
		}
		catch(IOException ioe){
		    ioe.printStackTrace();
		}
		finally{
		    try{
			buffer_reader.close();
			reader.close();
		    }
		    catch(IOException ioe_close){
			ioe_close.printStackTrace();
		    }
		}
	
		Document doc = text_panel.getDocument();
		try{
		    doc.remove(0,doc.getLength());
		    doc.insertString(0,
				     default_content,
				     null);
		    text_panel.setCaretPosition(0);
		}
		catch(BadLocationException ble){
		    ble.printStackTrace();
		}


	    }
	}


    }




}