///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.Hashtable;

//Class that will build the configuration window
public class Configuration_sequences_window 
    extends JDialog implements ActionListener{

    //Main_window
    Main_window main_window = null;

    //Main panel
    private JPanel main_conf_seq_panel = null;

    //Buttons
    private JButton button_save = null;
    private JButton button_cancel = null;

    //Layout and constraints
    private GridBagLayout gb_layout = null;
    private GridBagConstraints constraints = null;
 
    //Sequences/species to configure
    private String sequences;

    //Array of species probabilities
    private String[] seq_proba;
    
    //Array of sequences location
    private String[] sequences_path_array;

    //Hashtable that links the sequence to its probability
    private Hashtable<String,String> hash_proba;

    //Constructor
    public Configuration_sequences_window(Main_window main_window){

	super(main_window,"Sequences configuration",true);
	this.setLocationRelativeTo(main_window);
	this.setResizable(false);
	this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	this.main_window = main_window;
    } 
				

    //Build the dialog
    public void build(){

	//Layout and constraints definitions
	gb_layout = new GridBagLayout();
	constraints = new GridBagConstraints();
	constraints.insets = new Insets(10,10,10,10);


	//Main panel
	main_conf_seq_panel = new JPanel(gb_layout);
	main_conf_seq_panel.setBackground(Color.white);


	//Y axis current position
	int y_position = 0;

	//Sequences
	hash_proba = main_window.get_hash_proba();
	sequences = main_window.get_others_text();
	sequences_path_array = sequences.split(",");
	seq_proba = new String[sequences_path_array.length];

	for(int i = 0; i < sequences_path_array.length; i++){
	    File current_sequence = new File(sequences_path_array[i]);
	    String current_sequence_abs_path = 
		current_sequence.getAbsolutePath();
	    String[] sequences_name = current_sequence_abs_path.split("\\"+File.separator);
	    String current_name = sequences_name[sequences_name.length - 1];

	    //Check if the proba was already chosen
	    if(hash_proba.containsKey(sequences_path_array[i])){
		seq_proba[i] = (String)hash_proba.get(sequences_path_array[i]);
	    }
	    else{
		//Default values 
		seq_proba[i] = main_window.get_default_proba();
	    }

	    //Label for name
	    constraints.gridx = 0;
	    constraints.gridy = y_position;
	    constraints.gridwidth = 1;
	    constraints.gridheight = 1;
	    constraints.anchor = GridBagConstraints.LINE_START;
	    main_conf_seq_panel.add(new JLabel(current_name),constraints);

	    //Radio buttons
	    JRadioButton radio_default = new JRadioButton("Default");
	    radio_default.addActionListener(this);
	    if(seq_proba[i].equals(main_window.get_default_proba())){
		radio_default.setSelected(true);
	    }
	    radio_default.setBackground(Color.white);
	    radio_default.setActionCommand(i+":0.8");
	    JRadioButton radio_mandatory = new JRadioButton("Mandatory");
	    radio_mandatory.addActionListener(this);
	    if(seq_proba[i].equals("1.0")){
		radio_mandatory.setSelected(true);
	    }
	    radio_mandatory.setBackground(Color.white);
	    radio_mandatory.setActionCommand(i+":1.0"); 
	    JRadioButton radio_excluded = new JRadioButton("Excluded");
	    radio_excluded.addActionListener(this);
	    if(seq_proba[i].equals("0.0")){
		radio_excluded.setSelected(true);
	    }
	    radio_excluded.setBackground(Color.white);
	    radio_excluded.setActionCommand(i+":0.0");
	    

	    ButtonGroup group = new ButtonGroup();
	    group.add(radio_default);
	    group.add(radio_mandatory);
	    group.add(radio_excluded);
	    
	    constraints.gridx = 1;
	    constraints.gridy = y_position;
	    constraints.gridwidth = 1;
	    constraints.gridheight = 1;
	    constraints.anchor = GridBagConstraints.CENTER;
	    main_conf_seq_panel.add(radio_default,constraints);

	    constraints.gridx = 2;
	    constraints.gridy = y_position;
	    constraints.gridwidth = 1;
	    constraints.gridheight = 1;
	    main_conf_seq_panel.add(radio_mandatory,constraints);

	    constraints.gridx = 3;
	    constraints.gridy = y_position;
	    constraints.gridwidth = 1;
	    constraints.gridheight = 1;
	    main_conf_seq_panel.add(radio_excluded,constraints);

	    y_position++;
	}



	//Buttons
	button_save = new JButton("Save");
	button_save.addActionListener(this);
	constraints.gridx = 2;
	constraints.gridy = y_position;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	main_conf_seq_panel.add(button_save,constraints);

	button_cancel = new JButton("Cancel");
	button_cancel.addActionListener(this);
	constraints.gridx = 3;
	constraints.gridy = y_position;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	main_conf_seq_panel.add(button_cancel,constraints);
	
	getContentPane().add(main_conf_seq_panel);
	pack();
	setVisible(true);



    }


    public void actionPerformed(ActionEvent e){


	//Save button : save proba and close the dialog
	if (e.getSource() == button_save){

	    for(int i = 0; i < seq_proba.length; i++){
		
		hash_proba.put(sequences_path_array[i],seq_proba[i]);	

	    }
	    
	    main_window.set_species_proba(seq_proba);
	    main_window.set_hash_proba(hash_proba);
	    setVisible(false);

	}
	else{
	    //Cancel button : set old values
	    if(e.getSource() == button_cancel){
		for(int i = 0; i < seq_proba.length; i++){
		    if(hash_proba.containsKey(sequences_path_array[i])){
			seq_proba[i] = 
			    (String)hash_proba.get(sequences_path_array[i]);
		    }
		    else{
			seq_proba[i] = main_window.get_default_proba();
		    }
		}
		setVisible(false);

	    }
	    //radio buttons
	    else{
		String value = e.getActionCommand();
		String[] split_values = value.split(":");
		//First the index in the array and second the value
		seq_proba[Integer.parseInt(split_values[0])] = split_values[1];


	    }
	}

    }



    //Return sequences probabilities
    public String[] get_sequences_proba(){

	return seq_proba;

    }


}