///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

//Class that will build the main configuration window
public class Configuration_software_main_window 
    extends JDialog implements ActionListener{

    //Main window
    private Main_window main_window = null;

    //Main panel
    private JPanel main_conf_soft_panel = null;

    //Layout and constraints
    private GridBagLayout gb_layout = null;
    private GridBagConstraints constraints = null;

    //Label
    private JLabel label_software = null;

    //Combo box
    private JComboBox combo_box_soft = null;

    //Buttons
    private JButton button_ok = null;
    private JButton button_done = null;

    //Software to configure
    private String[] software;


    //Constructor
    public Configuration_software_main_window(Main_window main_window, 
					      String[] software){

	super(main_window,"Software configuration",true);
	this.main_window = main_window;
	this.setLocationRelativeTo(main_window);
	this.setResizable(false);
	this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	this.software = software;

    } 

    //Build the dialog
    public void build(){



	//Layout and constraints definitions
	gb_layout = new GridBagLayout();
	constraints = new GridBagConstraints();
	constraints.insets = new Insets(10,10,10,10);


	//Main panel
	main_conf_soft_panel = new JPanel(gb_layout);
	main_conf_soft_panel.setBackground(Color.white);

	//Label 
	label_software = new JLabel("Choose software to configure");
	constraints.gridx = 1;
	constraints.gridy = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	constraints.anchor = GridBagConstraints.CENTER;
	main_conf_soft_panel.add(label_software,constraints);
	

	//Combo box
	combo_box_soft = new JComboBox(software);
	combo_box_soft.setEditable(false);
	constraints.gridx = 1;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	main_conf_soft_panel.add(combo_box_soft,constraints);


	//Buttons
	button_ok = new JButton("OK");
	button_ok.addActionListener(this);
	constraints.gridx = 0;
	constraints.gridy = 2;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	main_conf_soft_panel.add(button_ok,constraints);

	button_done = new JButton("Close");
	button_done.addActionListener(this);
	constraints.gridx = 2;
	constraints.gridy = 2;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	main_conf_soft_panel.add(button_done,constraints);

	getContentPane().add(main_conf_soft_panel);
	pack();
	setVisible(true);

    }

    public void actionPerformed(ActionEvent e){


	//Ok button : display dialog according to the software selected
	if(e.getSource() == button_ok){

	    Configuration_software_window conf_window = 
		new Configuration_software_window(main_window,
						  this);

	    //Mask redundancy conf
	    if((combo_box_soft.getSelectedItem()).equals("clean up redundancy")){
		conf_window.setTitle("Configuration of the redundancy clean up software");
		conf_window.build_mask_redundancy();
		main_window.set_masking_redundancy_conf(conf_window.get_new_conf_file());
	    }

	    //Yass conf
	    if((combo_box_soft.getSelectedItem()).equals("yass")){
		conf_window.setTitle("Configuration of the Yass software");
		conf_window.build_yass();
		main_window.set_yass_conf(conf_window.get_new_conf_file());
	    }

	    //Blast conf
	    if((combo_box_soft.getSelectedItem()).equals("blast")){
		conf_window.setTitle("Configuration of the Blast software");
		conf_window.build_blast();
		main_window.set_blast_conf(conf_window.get_new_conf_file());
	    }

	    //CG-seq conf
	    if((combo_box_soft.getSelectedItem()).equals("conserved regions")){
		conf_window.setTitle("Configuration of conserved regions");
		conf_window.build_cg_seq();
		main_window.set_cg_seq_conf(conf_window.get_new_conf_file());
	    }

	    //Carnac conf
	    if((combo_box_soft.getSelectedItem()).equals("carnac")){
		conf_window.setTitle("Configuration of the Carnac software");
		conf_window.build_carnac();
		main_window.set_carnac_conf(conf_window.get_new_conf_file());
	    }

	    //RNAz conf
	    if((combo_box_soft.getSelectedItem()).equals("rnaz")){
		conf_window.setTitle("Configuration of the RNAz software");
		conf_window.build_rnaz();
		main_window.set_rnaz_conf(conf_window.get_new_conf_file());
	    }


	}
	//Done button : return to main window
	else{
	    setVisible(false);


	}


    }


    //Get the item slectionned in the combo box for software choose
    public String get_selected_soft(){

	return (String)combo_box_soft.getSelectedItem();
    }


}
