///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JRadioButton;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.ButtonGroup;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.regex.Pattern;
import java.util.regex.Matcher;


//Class that will build the configuration window
public class Configuration_software_window 
    extends JDialog implements ActionListener{

    //Main window
    Main_window main_window = null;

    //Main configuration window
    Configuration_software_main_window main_conf_window = null;

    //Main panel
    private JPanel soft_panel = null;

    //Layout and constraints
    private GridBagLayout gb_layout = null;
    private GridBagConstraints constraints = null;

    //Buttons
    private JButton button_save = null;
    private JButton button_cancel = null;
    private JButton button_default = null;
    private JButton button_advanced = null;

    //Dimensions fortextfields
    private Dimension small_dimension = null;
    private Dimension large_dimension = null;

    //Content of the current configuration file
    private String content;

    //Content of the default configuration file
    private String default_content;

    //Array of parameters regexp:values (e.i --mode\s[01]:1)
    //that will be saved for the new configuration
    private String[] parameters;

    //Current configuration file
    private String current_conf_file;

    //File where to write the new configuration
    private String new_conf_file;

    //Default configuration file
    private String default_conf_file;
    
     //Masking redundancy regular expression, values and components
    private String masking_redundancy_length_value;
    private String masking_redundancy_length_regexp;
    private String masking_redundancy_id_value;
    private String masking_redundancy_id_regexp;
    private JTextField masking_redundancy_length_text = null;
    private JTextField masking_redundancy_id_text = null;

    //Yass regular expressions, values and components
    private String yass_strand_regexp;
    private String yass_strand_value;
    private String yass_evalue_regexp;
    private String yass_evalue_value;
    private String yass_costs_regexp;
    private String yass_costs_value;
    private String yass_gap_costs_regexp;
    private String yass_gap_costs_value;
    private String yass_seed_regexp;
    private String yass_seed_value;
    private String yass_hit_criterion_regexp;
    private String yass_hit_criterion_value;
    private String yass_low_complexity_regexp;
    private String yass_low_complexity_value;
    private JRadioButton yass_radio_strand_both = null;
    private JRadioButton yass_radio_strand_forward = null;
    private JRadioButton yass_radio_strand_reverse = null;
    private JTextField yass_evalue_text = null;
    private JTextField yass_costs_text = null;
    private JTextField yass_gap_text = null;
    private JTextField yass_seed_text = null;
    private JRadioButton yass_radio_hit_simple = null;
    private JRadioButton yass_radio_hit_double = null;
    private JRadioButton yass_radio_filter_yes = null;
    private JRadioButton yass_radio_filter_no = null;


    //Blast regular expressions, values and components
    private String blast_strand_regexp;
    private String blast_strand_value;
    private String blast_evalue_regexp;
    private String blast_evalue_value;
    private String blast_word_regexp;
    private String blast_word_value;
    private String blast_match_regexp ;    
    private String blast_match_value;
    private String blast_mismatch_regexp;   
    private String blast_mismatch_value;    
    private String blast_gap_open_regexp;  
    private String blast_gap_open_value;    
    private String blast_gap_ext_regexp;
    private String blast_gap_ext_value;
    private String blast_low_complexity_regexp;
    private String blast_low_complexity_value;
    private JRadioButton blast_radio_strand_both = null;
    private JRadioButton blast_radio_strand_forward = null;
    private JRadioButton blast_radio_strand_reverse = null;
    private JTextField blast_evalue_text = null;
    private JTextField blast_word_text = null;
    private JComboBox blast_costs_combo_box = null;
    private JRadioButton blast_radio_filter_yes = null;
    private JRadioButton blast_radio_filter_no = null;


    //CG-seq regular expressions, values and components
    private String cg_seq_min_id_regexp;
    private String cg_seq_min_id_value;
    private String cg_seq_max_id_regexp;
    private String cg_seq_max_id_value;
    private String cg_seq_mode_regexp;
    private String cg_seq_mode_value;
    private String cg_seq_max_length_regexp;
    private String cg_seq_max_length_value;
    private String cg_seq_min_length_regexp;
    private String cg_seq_min_length_value;
    private String cg_seq_nb_seqs_regexp;
    private String cg_seq_nb_seqs_value;
    private JTextField cg_seq_min_id_text = null;
    private JTextField cg_seq_max_id_text = null;
    private JTextField cg_seq_min_length_text = null;
    private JTextField cg_seq_max_length_text = null;
    private JRadioButton cg_seq_mode_star = null;
    private JRadioButton cg_seq_mode_naive = null;
    private JRadioButton cg_seq_mode_clique = null;
    private JTextField cg_seq_nb_seqs_text = null;


    //Carnac regular expression, value and components
    private String carnac_gc_regexp_used;
    private String carnac_gc_value_used; 
    private JRadioButton carnac_gc_yes = null;
    private JRadioButton carnac_gc_no = null;


    //RNAz regular expression, value and components
    private String rnaz_proba_regexp;
    private String rnaz_proba_value;
    private JTextField rnaz_proba_text = null;



    //Constructor
    public Configuration_software_window(Main_window main_window,
					 Configuration_software_main_window main_conf_window){

	super(main_conf_window,true);
	this.main_window = main_window;
	this.main_conf_window = main_conf_window;
	this.setLocationRelativeTo(main_conf_window);
	this.setResizable(false);
	this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

	//Layout and constraints definitions
	gb_layout = new GridBagLayout();
	constraints = new GridBagConstraints();
	constraints.insets = new Insets(10,10,10,10);
	constraints.anchor = GridBagConstraints.LINE_START;
	constraints.gridheight = 1;

	//Main panel
	soft_panel = new JPanel(gb_layout);
	soft_panel.setBackground(Color.white);

	//Dimensions fortextfields
	small_dimension = new Dimension(100,35);
	large_dimension = new Dimension(400,35);

    } 



    //Build the dialog for masking redundancy configuration
    public void build_mask_redundancy(){


	//Get the content of the masking configuration file
	current_conf_file = main_window.get_masking_redundancy_conf();
	content = main_window.get_conf_content(current_conf_file);

	//Get future location of the new masking configuration file
	new_conf_file = main_window.get_temp_masking_redundancy_conf();

	//Get the default configuration file
	default_conf_file = main_window.get_default_masking_redundancy_conf();

	//Regular expression matching length
	masking_redundancy_length_regexp = "(--length[ \\t]+)([0-9]+)";

	//Regular expression matching identity
	masking_redundancy_id_regexp = "(--id[ \\t]+)([0-9\\.]+)";

	//Get the current masking mode 
	masking_redundancy_length_value = 
	    get_parameter_value(content,
				masking_redundancy_length_regexp);

	//Get the current masking mode 
	masking_redundancy_id_value = 
	    get_parameter_value(content,
				masking_redundancy_id_regexp);


	//Set the array of parameters
	parameters = new String[2];
	parameters[0] = 
	    masking_redundancy_length_regexp+":"+masking_redundancy_length_value;
	parameters[1] = 
	    masking_redundancy_id_regexp+":"+masking_redundancy_id_value;

	//Length parameter
	JLabel mask_redundancy_length_label = 
	    new JLabel("Minimum length of a region");
	place_component(mask_redundancy_length_label,0,0,2);

	masking_redundancy_length_text = new JTextField();
	masking_redundancy_length_text.setPreferredSize(small_dimension);
	masking_redundancy_length_text.setMinimumSize(small_dimension);
	masking_redundancy_length_text.setText(masking_redundancy_length_value);
	masking_redundancy_length_text.addFocusListener(new Focus_listener(this,0,false));
	place_component(masking_redundancy_length_text,2,0,3);

	//Identity parameter
	JLabel mask_redundancy_id_label = 
	    new JLabel("Minimum identity to consider redundancy");
	place_component(mask_redundancy_id_label,0,1,2);

	masking_redundancy_id_text = new JTextField();
	masking_redundancy_id_text.setPreferredSize(small_dimension);
	masking_redundancy_id_text.setMinimumSize(small_dimension);
	masking_redundancy_id_text.setText(masking_redundancy_id_value);
	masking_redundancy_id_text.addFocusListener(new Focus_listener(this,1,false));
	place_component(masking_redundancy_id_text,2,1,3);

	place_buttons(3);

		
	getContentPane().add(soft_panel);
	pack();
	setVisible(true);


    }


    //Build the dialog for Yass configuration
    public void build_yass(){

	//Get the content of yass configuration file
	current_conf_file = main_window.get_yass_conf();
	content = main_window.get_conf_content(current_conf_file);

	//Get future location of the new yass configuration file
	new_conf_file = main_window.get_temp_yass_conf();


	//Get the default configuration file
	default_conf_file = main_window.get_default_yass_conf();

	//Regular expression for the query strand processing
	yass_strand_regexp = "(-r[ \\t]+)([012])";
	//Get the current strand
	yass_strand_value = get_parameter_value(content,
						yass_strand_regexp);
	//Regular expression for the evalue
	yass_evalue_regexp = "(-E[ \\t]+)(\\d*\\.?\\d+(e\\-?\\d+)?)";
	//Get the current gap costs
	yass_evalue_value = get_parameter_value(content,
						yass_evalue_regexp);
	//Regular expression for the costs
	yass_costs_regexp = 
	    "(-C[ \\t]+)(\\-?\\d+,\\-?\\d+,\\-?\\d+,\\-?\\d+)";
	//Get the current costs
	yass_costs_value = get_parameter_value(content,
					       yass_costs_regexp);

	//Regular expression for the gap costs
	yass_gap_costs_regexp = "(-G[ \\t]+)(\\-?\\d+,\\-?\\d+)";
	//Get the current gap costs
	yass_gap_costs_value = get_parameter_value(content,
						   yass_gap_costs_regexp);

	//Regular expression for the seeds
	yass_seed_regexp = "(\\n\\-p[ \\t]+)\\\"([#\\-@,]+)\\\"";
	//Get the current seed
	yass_seed_value = get_parameter_value(content,
					      yass_seed_regexp);
	
	//Regular expression for the seed hit criterion
	yass_hit_criterion_regexp = "(-c[ \\t]+)([12])";
	//Get the current hit criterion
	yass_hit_criterion_value = 
	    get_parameter_value(content,
				yass_hit_criterion_regexp);

	//Regular expression for the filter of low complexity
	yass_low_complexity_regexp = "(-e[ \\t]+)([\\d\\.]+)";
	//Get the current low complexity filter
	yass_low_complexity_value = 
	    get_parameter_value(content,
				yass_low_complexity_regexp);


	//Set the array of parameters
	parameters = new String[7];
	parameters[0] = yass_strand_regexp+":"+yass_strand_value;
	parameters[1] = yass_evalue_regexp+":"+yass_evalue_value;
	parameters[2] = yass_costs_regexp+":"+yass_costs_value;
	parameters[3] = yass_gap_costs_regexp+":"+yass_gap_costs_value;
	parameters[4] = yass_seed_regexp+":"+"\""+yass_seed_value+"\"";
	parameters[5] = yass_hit_criterion_regexp+":"+yass_hit_criterion_value;
	parameters[6] = 
	    yass_low_complexity_regexp+":"+yass_low_complexity_value;


	//Strand parameter
	JLabel yass_strand_label = new JLabel("Strand to process");
	place_component(yass_strand_label,0,0,2);
	
	yass_radio_strand_both = new JRadioButton("Both");
	yass_radio_strand_both.addActionListener(this);
	if(yass_strand_value.equals("2")){
	    yass_radio_strand_both.setSelected(true);
	}
	yass_radio_strand_both.setBackground(Color.white);
	yass_radio_strand_both.setActionCommand("0:2");
	
	yass_radio_strand_forward = new JRadioButton("Forward");
	yass_radio_strand_forward.addActionListener(this);
	if(yass_strand_value.equals("0")){
	    yass_radio_strand_forward.setSelected(true);
	}
	yass_radio_strand_forward.setBackground(Color.white);
	yass_radio_strand_forward.setActionCommand("0:0");
	
	yass_radio_strand_reverse = new JRadioButton("Reverse");
	yass_radio_strand_reverse.addActionListener(this);
	if(yass_strand_value.equals("1")){
	    yass_radio_strand_reverse.setSelected(true);
	}
	yass_radio_strand_reverse.setBackground(Color.white);
	yass_radio_strand_reverse.setActionCommand("0:1");

	ButtonGroup yass_group_strand = new ButtonGroup();
	yass_group_strand.add(yass_radio_strand_both);
	yass_group_strand.add(yass_radio_strand_forward);
	yass_group_strand.add(yass_radio_strand_reverse);

	place_component(yass_radio_strand_both,2,0,1);
	place_component(yass_radio_strand_forward,3,0,1);
	place_component(yass_radio_strand_reverse,4,0,1);


	//Evalue parameter
	JLabel yass_evalue_label = new JLabel("E-value threshold");
	place_component(yass_evalue_label,0,1,2);

	yass_evalue_text = new JTextField();
	yass_evalue_text.setPreferredSize(small_dimension);
	yass_evalue_text.setMinimumSize(small_dimension);
	yass_evalue_text.setText(yass_evalue_value);
	yass_evalue_text.addFocusListener(new Focus_listener(this,1,false));
	place_component(yass_evalue_text,2,1,3);

	//Match/transition/transversion/other costs
	JLabel yass_costs_label = 
	    new JLabel("Match/transition/transversion/other costs");
	place_component(yass_costs_label,0,2,2);

	yass_costs_text = new JTextField();
	yass_costs_text.setPreferredSize(small_dimension);
	yass_costs_text.setMinimumSize(small_dimension);
	yass_costs_text.setText(yass_costs_value);
	yass_costs_text.addFocusListener(new Focus_listener(this,2,false));
	place_component(yass_costs_text,2,2,3);
	
	//Gap opening and extension costs
	JLabel yass_gap_label = 
	    new JLabel("Gap opening and extension costs");
	place_component(yass_gap_label,0,3,2);

	yass_gap_text = new JTextField();
	yass_gap_text.setPreferredSize(small_dimension);
	yass_gap_text.setMinimumSize(small_dimension);
	yass_gap_text.setText(yass_gap_costs_value);
	yass_gap_text.addFocusListener(new Focus_listener(this,3,false));
	place_component(yass_gap_text,2,3,3);


	//Seed pattern
	JLabel yass_seed_label = 
	    new JLabel("<html>Seed pattern (\"#\":match; \"@\":match or transition;<br>\"_\":joker; \",\":seed separator)</html>");
	place_component(yass_seed_label,0,4,2);

	yass_seed_text = new JTextField();
	yass_seed_text.setPreferredSize(large_dimension);
	yass_seed_text.setMinimumSize(small_dimension);
	yass_seed_text.setText(yass_seed_value);
	yass_seed_text.addFocusListener(new Focus_listener(this,4,true));
	place_component(yass_seed_text,2,4,3);


	//Seed hit criterion
	JLabel yass_hit_criterion_label = new JLabel("Seed hit criterion");
	place_component(yass_hit_criterion_label,0,5,2);
	
	yass_radio_hit_simple = new JRadioButton("Simple");
	yass_radio_hit_simple.addActionListener(this);
	yass_radio_hit_simple.setActionCommand("5:1");
	if(yass_hit_criterion_value.equals("1")){
	    yass_radio_hit_simple.setSelected(true);
	}
	yass_radio_hit_simple.setBackground(Color.white);
	
	yass_radio_hit_double = new JRadioButton("Double");
	yass_radio_hit_double.addActionListener(this);
	yass_radio_hit_double.setActionCommand("5:2");
	if(yass_hit_criterion_value.equals("2")){
	    yass_radio_hit_double.setSelected(true);
	}
	yass_radio_hit_double.setBackground(Color.white);

	ButtonGroup yass_group_hit = new ButtonGroup();
	yass_group_hit.add(yass_radio_hit_simple);
	yass_group_hit.add(yass_radio_hit_double);

	place_component(yass_radio_hit_simple,2,5,1);
	place_component(yass_radio_hit_double,3,5,1);		

	//Low complexity filter
	JLabel yass_low_complexity_label = new JLabel("Low complexity filter");
	place_component(yass_low_complexity_label,0,6,2);
	
	yass_radio_filter_yes = new JRadioButton("Yes");
	yass_radio_filter_yes.addActionListener(this);
	if(!(yass_low_complexity_value.equals("0"))){
	    yass_radio_filter_yes.setSelected(true);
	}
	yass_radio_filter_yes.setBackground(Color.white);
	yass_radio_filter_yes.setActionCommand("6:2.8");

	yass_radio_filter_no = new JRadioButton("No");
	yass_radio_filter_no.addActionListener(this);
	if(yass_low_complexity_value.equals("0")){
	    yass_radio_filter_no.setSelected(true);
	}
	yass_radio_filter_no.setBackground(Color.white);
	yass_radio_filter_no.setActionCommand("6:0");

	ButtonGroup yass_group_filter = new ButtonGroup();
	yass_group_filter.add(yass_radio_filter_yes);
	yass_group_filter.add(yass_radio_filter_no);

	place_component(yass_radio_filter_yes,2,6,1);
	place_component(yass_radio_filter_no,3,6,1);
	//Buttons
	place_buttons(7);

		
	getContentPane().add(soft_panel);
	pack();
	setVisible(true);

    }


    //Build the dialog for Blast configuration
    public void build_blast(){

	//Get the content of the blast configuration file
	current_conf_file = main_window.get_blast_conf();
	content = main_window.get_conf_content(current_conf_file);

	//Get future location of the new blast configuration file
	new_conf_file = main_window.get_temp_blast_conf();

	//Get the default configuration file
	default_conf_file = main_window.get_default_blast_conf();

	//Regular expression for the query strand processing
	blast_strand_regexp = "(-S[ \\t]+)([123])";
	//Get the current strand
	blast_strand_value = get_parameter_value(content,
						 blast_strand_regexp);

	//Regular expression for the evalue
	blast_evalue_regexp = "(-e[ \\t]+)(\\d*\\.?\\d+(e\\-?\\d+)?)";
	//Get the current evalue
	blast_evalue_value = get_parameter_value(content,
						 blast_evalue_regexp);

	//Regular expression for the word size
	blast_word_regexp = "(-W[ \\t]+)([\\d]+)";
	//Get the current word size
	blast_word_value = get_parameter_value(content,
					       blast_word_regexp);
	
	//Array of possible costs
	String[] blast_possible_costs = {"2; -7; 2; 4",
					 "2; -7; 0; 4",
					 "2; -7; 4; 2",
					 "2; -7; 2; 2",
					 "2; -7; 4; 4",
					 "4; -5; 6; 5",
					 "4; -5; 5; 5",
					 "4; -5; 4; 5",
					 "4; -5; 3; 5",
					 "4; -5; 12; 8",
					 "2; -5; 2; 4",
					 "2; -5; 0; 4",
					 "2; -5; 4; 2",
					 "2; -5; 2; 2",
					 "2; -5; 4; 4",
					 "1; -2; 1; 2",
					 "1; -2; 0; 2",
					 "1; -2; 3; 1",
					 "1; -2; 2; 1",
					 "1; -2; 1; 1",
					 "1; -2; 2; 2",
					 "1; -3; 1; 2",
					 "1; -3; 0; 2",
					 "1; -3; 2; 1",
					 "1; -3; 1; 1",
					 "1; -3; 2; 2",
					 "2; -3; 4; 4",
					 "2; -3; 2; 4",
					 "2; -3; 0; 4",
					 "2; -3; 3; 3",
					 "2; -3; 6; 2",
					 "2; -3; 5; 2",
					 "2; -3; 4; 2",
					 "2; -3; 2; 2",
					 "2; -3; 6; 4",
					 "1; -1; 3; 2",
					 "1; -1; 2; 2",
					 "1; -1; 1; 2",
					 "1; -1; 0; 2",
					 "1; -1; 4; 1",
					 "1; -1; 3; 1",
					 "1; -1; 2; 1",
					 "1; -1; 4; 2",
					 "5; -4; 10; 6",
					 "5; -4; 8; 6",
					 "5; -4; 25; 10"};
	
	//Regular expression for the match reward
	blast_match_regexp = "(-r[ \\t]+)([\\d]+)";
	//Get the current match reward
	blast_match_value = get_parameter_value(content,
						blast_match_regexp);
	//Regular expression for the mismatch penalty
	blast_mismatch_regexp = "(-q[ \\t]+)([\\-\\d]+)";
	//Get the current mismatch penalty
	blast_mismatch_value = get_parameter_value(content,
						   blast_mismatch_regexp);
	//Regular expression for the gap opening penalty
	blast_gap_open_regexp = "(-G[ \\t]+)([\\-\\d]+)";
	//Get the current the gap opening penalty
	blast_gap_open_value = get_parameter_value(content,
						   blast_gap_open_regexp);
	//Regular expression for the gap extension penalty
	blast_gap_ext_regexp = "(-E[ \\t]+)([\\-\\d]+)";
	//Get the current the gap extension penalty
	blast_gap_ext_value = get_parameter_value(content,
						  blast_gap_ext_regexp);

	//All costs parameters above in one 
	String blast_all_costs_value = blast_match_value+
	    "; "+blast_mismatch_value+
	    "; "+blast_gap_open_value+
	    "; "+blast_gap_ext_value;

	
	//Regular expression for the filter of low complexity
	blast_low_complexity_regexp = "(-F[ \\t]+)([FT])";
	//Get the current low complexity filter
	blast_low_complexity_value = 
	    get_parameter_value(content,
				blast_low_complexity_regexp);



	//Set the array of parameters
	parameters = new String[8];
	parameters[0] = blast_strand_regexp+":"+blast_strand_value;
	parameters[1] = blast_evalue_regexp+":"+blast_evalue_value;
	parameters[2] = blast_word_regexp+":"+blast_word_value;
	parameters[3] = blast_match_regexp+":"+blast_match_value;
	parameters[4] = blast_mismatch_regexp+":"+blast_mismatch_value;
	parameters[5] = blast_gap_open_regexp+":"+blast_gap_open_value;
	parameters[6] = blast_gap_ext_regexp+":"+blast_gap_ext_value;
	parameters[7] = 
	    blast_low_complexity_regexp+":"+blast_low_complexity_value;



	//Strand parameter
	JLabel blast_strand_label = new JLabel("Strand to process");
	place_component(blast_strand_label,0,0,2);
	
	blast_radio_strand_both = new JRadioButton("Both");
	blast_radio_strand_both.addActionListener(this);
	if(blast_strand_value.equals("3")){
	    blast_radio_strand_both.setSelected(true);
	}
	blast_radio_strand_both.setBackground(Color.white);
	blast_radio_strand_both.setActionCommand("0:3");
	
	blast_radio_strand_forward = new JRadioButton("Forward");
	blast_radio_strand_forward.addActionListener(this);
	if(blast_strand_value.equals("1")){
	    blast_radio_strand_forward.setSelected(true);
	}
	blast_radio_strand_forward.setBackground(Color.white);
	blast_radio_strand_forward.setActionCommand("0:1");
	
	blast_radio_strand_reverse = new JRadioButton("Reverse");
	blast_radio_strand_reverse.addActionListener(this);
	if(blast_strand_value.equals("2")){
	    blast_radio_strand_reverse.setSelected(true);
	}
	blast_radio_strand_reverse.setBackground(Color.white);
	blast_radio_strand_reverse.setActionCommand("0:2");

	ButtonGroup blast_group_strand = new ButtonGroup();
	blast_group_strand.add(blast_radio_strand_both);
	blast_group_strand.add(blast_radio_strand_forward);
	blast_group_strand.add(blast_radio_strand_reverse);

	place_component(blast_radio_strand_both,2,0,1);
	place_component(blast_radio_strand_forward,3,0,1);
	place_component(blast_radio_strand_reverse,4,0,1);

	//Evalue parameter
	JLabel blast_evalue_label = new JLabel("E-value threshold");
	place_component(blast_evalue_label,0,1,2);

	blast_evalue_text = new JTextField();
	blast_evalue_text.setPreferredSize(small_dimension);
	blast_evalue_text.setMinimumSize(small_dimension);
	blast_evalue_text.setText(blast_evalue_value);
	blast_evalue_text.addFocusListener(new Focus_listener(this,1,false));
	place_component(blast_evalue_text,2,1,3);

	//Word size parameter
	JLabel blast_word_label = new JLabel("Word size (7-30)");
	place_component(blast_word_label,0,2,2);

	blast_word_text = new JTextField();
	blast_word_text.setPreferredSize(small_dimension);
	blast_word_text.setMinimumSize(small_dimension);
	blast_word_text.setText(blast_word_value);
	blast_word_text.addFocusListener(new Focus_listener(this,2,false));
	place_component(blast_word_text,2,2,3);

	//Costs parameters
	JLabel blast_costs_label = 
	    new JLabel("Match/mismatch/gap opening/gap extension costs");
	place_component(blast_costs_label,0,3,2);

	blast_costs_combo_box = new JComboBox(blast_possible_costs);
	blast_costs_combo_box.setEditable(false);
	blast_costs_combo_box.setSelectedItem(blast_all_costs_value);
	blast_costs_combo_box.setActionCommand("3:4:5:6");
	blast_costs_combo_box.addActionListener(this);
	place_component(blast_costs_combo_box,2,3,1);


	//Low complexity filter
	JLabel blast_low_complexity_label = 
	    new JLabel("Low complexity filter");
	place_component(blast_low_complexity_label,0,4,2);
	
	blast_radio_filter_yes = new JRadioButton("Yes");
	blast_radio_filter_yes.addActionListener(this);
	if(blast_low_complexity_value.equals("T")){
	    blast_radio_filter_yes.setSelected(true);
	}
	blast_radio_filter_yes.setBackground(Color.white);
	blast_radio_filter_yes.setActionCommand("7:T");

	blast_radio_filter_no = new JRadioButton("No");
	blast_radio_filter_no.addActionListener(this);
	if(blast_low_complexity_value.equals("F")){
	    blast_radio_filter_no.setSelected(true);
	}
	blast_radio_filter_no.setBackground(Color.white);
	blast_radio_filter_no.setActionCommand("7:F");

	ButtonGroup blast_group_filter = new ButtonGroup();
	blast_group_filter.add(blast_radio_filter_yes);
	blast_group_filter.add(blast_radio_filter_no);

	place_component(blast_radio_filter_yes,2,4,1);
	place_component(blast_radio_filter_no,3,4,1);

	place_buttons(5);


	getContentPane().add(soft_panel);
	pack();
	setVisible(true);

    }
	

    

    //Build the dialog for Cg_Seq configuration
    public void build_cg_seq(){

	//Get the content of cg_seq configuration file
	current_conf_file = main_window.get_cg_seq_conf();
	content = main_window.get_conf_content(current_conf_file);

	//Get future location of the new cg_seq configuration file
	new_conf_file = main_window.get_temp_cg_seq_conf();

	//Get the default configuration file
	default_conf_file = main_window.get_default_cg_seq_conf();

	//Regular expression for the minimum identity parameter
	cg_seq_min_id_regexp = "(-i[ \\t]+)(\\d+)";
	//Get the current minimum identity
	cg_seq_min_id_value = get_parameter_value(content,
						cg_seq_min_id_regexp);

	//Regular expression for the maximum identity parameter
	cg_seq_max_id_regexp = "(-I[ \\t]+)(\\d+)";
	//Get the current maximum identity
	cg_seq_max_id_value = get_parameter_value(content,
						cg_seq_max_id_regexp);


	//Regular expression for the mode selection parameter
	cg_seq_mode_regexp = "(-M[ \\t]+)([012])";
	//Get the current mode selection
	cg_seq_mode_value = get_parameter_value(content,
					      cg_seq_mode_regexp);

	//Regular expression for the maximal length parameter
	cg_seq_max_length_regexp = "(-L[ \\t]+)(\\d+)";
	//Get the current maximal length selection
	cg_seq_max_length_value = get_parameter_value(content,
						    cg_seq_max_length_regexp);

	//Regular expression for the minimal length parameter
	cg_seq_min_length_regexp = "(-l[ \\t]+)(\\d+)";
	//Get the current minimal length selection
	cg_seq_min_length_value = get_parameter_value(content,
						    cg_seq_min_length_regexp);

	//Regular expression for the number of sequences parameter
	cg_seq_nb_seqs_regexp = "(-m[ \\t]+)(\\d+)";
	//Get tcg_seq_he current number of sequences selection
	cg_seq_nb_seqs_value = get_parameter_value(content,
					    cg_seq_nb_seqs_regexp);


	//Set the array of parameters
	parameters = new String[6];
	parameters[0] = cg_seq_min_id_regexp+":"+cg_seq_min_id_value;
	parameters[1] = cg_seq_max_id_regexp+":"+cg_seq_max_id_value;
	parameters[2] = cg_seq_min_length_regexp+":"+cg_seq_min_length_value;
	parameters[3] = cg_seq_max_length_regexp+":"+cg_seq_max_length_value;
	parameters[4] = cg_seq_mode_regexp+":"+cg_seq_mode_value;
	parameters[5] = cg_seq_nb_seqs_regexp+":"+cg_seq_nb_seqs_value;


	//Minimum identity parameter
	JLabel cg_seq_min_id_label = 
	    new JLabel("Minimum identity threshold (%)");
	place_component(cg_seq_min_id_label,0,0,2);

	cg_seq_min_id_text = new JTextField();
	cg_seq_min_id_text.setPreferredSize(small_dimension);
	cg_seq_min_id_text.setMinimumSize(small_dimension);
	cg_seq_min_id_text.setText(cg_seq_min_id_value);
	cg_seq_min_id_text.addFocusListener(new Focus_listener(this,0,false));
	place_component(cg_seq_min_id_text,2,0,3);

	//Maximum identity parameter
	JLabel cg_seq_max_id_label = 
	    new JLabel("Maximum identity threshold (%)");
	place_component(cg_seq_max_id_label,0,1,2);

	cg_seq_max_id_text = new JTextField();
	cg_seq_max_id_text.setPreferredSize(small_dimension);
	cg_seq_max_id_text.setMinimumSize(small_dimension);
	cg_seq_max_id_text.setText(cg_seq_max_id_value);
	cg_seq_max_id_text.addFocusListener(new Focus_listener(this,1,false));
	place_component(cg_seq_max_id_text,2,1,3);

	//Minimum length parameter
	JLabel cg_seq_min_length_label = new JLabel("Minimum length");
	place_component(cg_seq_min_length_label,0,2,2);

	cg_seq_min_length_text = new JTextField();
	cg_seq_min_length_text.setPreferredSize(small_dimension);
	cg_seq_min_length_text.setMinimumSize(small_dimension);
	cg_seq_min_length_text.setText(cg_seq_min_length_value);
	cg_seq_min_length_text.addFocusListener(new Focus_listener(this,2,false));
	place_component(cg_seq_min_length_text,2,2,3);

	//Maximum length parameter
	JLabel cg_seq_max_length_label = new JLabel("Maximum length");
	place_component(cg_seq_max_length_label,0,3,2);

	cg_seq_max_length_text = new JTextField();
	cg_seq_max_length_text.setPreferredSize(small_dimension);
	cg_seq_max_length_text.setMinimumSize(small_dimension);
	cg_seq_max_length_text.setText(cg_seq_max_length_value);
	cg_seq_max_length_text.addFocusListener(new Focus_listener(this,3,false));
	place_component(cg_seq_max_length_text,2,3,3);

	//Selection mode
	JLabel cg_seq_mode_label = new JLabel("Selection mode");
	place_component(cg_seq_mode_label,0,4,2);
	
	cg_seq_mode_star = new JRadioButton("Star");
	cg_seq_mode_star.addActionListener(this);
	if(cg_seq_mode_value.equals("0")){
	    cg_seq_mode_star.setSelected(true);
	}
	cg_seq_mode_star.setBackground(Color.white);
	cg_seq_mode_star.setActionCommand("4:0");

	cg_seq_mode_naive = new JRadioButton("Naive clique");
	cg_seq_mode_naive.addActionListener(this);
	if(cg_seq_mode_value.equals("1")){
	    cg_seq_mode_naive.setSelected(true);
	}
	cg_seq_mode_naive.setBackground(Color.white);
	cg_seq_mode_naive.setActionCommand("4:1");

	cg_seq_mode_clique = new JRadioButton("Maxi clique");
	cg_seq_mode_clique.addActionListener(this);
	if(cg_seq_mode_value.equals("2")){
	    cg_seq_mode_clique.setSelected(true);
	}
	cg_seq_mode_clique.setBackground(Color.white);
	cg_seq_mode_clique.setActionCommand("4:2");

	ButtonGroup cg_seq_group_mode = new ButtonGroup();
	cg_seq_group_mode.add(cg_seq_mode_star);
	cg_seq_group_mode.add(cg_seq_mode_naive);
	cg_seq_group_mode.add(cg_seq_mode_clique);
	
	place_component(cg_seq_mode_star,2,4,1);
	place_component(cg_seq_mode_naive,3,4,1);
	place_component(cg_seq_mode_clique,4,4,1);

	//Number of sequences parameter
	JLabel cg_seq_nb_seqs_label = 
	    new JLabel("Maximum number of sequences by set");
	place_component(cg_seq_nb_seqs_label,0,5,2);

	cg_seq_nb_seqs_text = new JTextField();
	cg_seq_nb_seqs_text.setPreferredSize(small_dimension);
	cg_seq_nb_seqs_text.setMinimumSize(small_dimension);
	cg_seq_nb_seqs_text.setText(cg_seq_nb_seqs_value);
	cg_seq_nb_seqs_text.addFocusListener(new Focus_listener(this,5,false));
	place_component(cg_seq_nb_seqs_text,2,5,3);

	place_buttons(6);


	getContentPane().add(soft_panel);
	pack();
	setVisible(true);
	


    }


    
    //Build the dialog for carnac configuration
    public void build_carnac(){

	//Get the content of carnac configuration file
	current_conf_file = main_window.get_carnac_conf();
	content = main_window.get_conf_content(current_conf_file);

	//Get future location of the new carnac configuration file
	new_conf_file = main_window.get_temp_carnac_conf();

	//Get the default configuration file
	default_conf_file = main_window.get_default_carnac_conf();

	//Regular expression for the GC % correction parameter being used
	carnac_gc_regexp_used = "(\\n[^#][ \\t]*)(\\-A)[ \\t]*\\n";
	//Get the current gc % percent parameter
	carnac_gc_value_used = get_parameter_value(content,
						   carnac_gc_regexp_used);

	//Check whether that option is used or not
	boolean carnac_gc_used = carnac_gc_value_used.equals("-A");

	//Set the array of parameters
	parameters = new String[1];

	if(carnac_gc_used){
	    parameters[0] = "(\\n)(#?[ \\t]*\\-A)[ \\t]*\\n"+":"+"-A\n";
	}
	else{
	    parameters[0] = "(\\n)(#?[ \\t]*\\-A)[ \\t]*\\n"+":"+"#-A\n";
	}

	//GC% mode
	JLabel carnac_gc_label = 
	    new JLabel("Disable threshold according to GC%");
	place_component(carnac_gc_label,0,0,2);
	
	carnac_gc_yes = new JRadioButton("Yes");
	carnac_gc_yes.addActionListener(this);
	if(carnac_gc_used){
	    carnac_gc_yes.setSelected(true);
	}
	carnac_gc_yes.setBackground(Color.white);
	carnac_gc_yes.setActionCommand("0:-A\n");

	carnac_gc_no = new JRadioButton("No");
	carnac_gc_no.addActionListener(this);
	if(!carnac_gc_used){
	    carnac_gc_no.setSelected(true);
	}
	carnac_gc_no.setBackground(Color.white);
	carnac_gc_no.setActionCommand("0:#-A\n");

	ButtonGroup carnac_group_gc = new ButtonGroup();
	carnac_group_gc.add(carnac_gc_yes);
	carnac_group_gc.add(carnac_gc_no);

	place_component(carnac_gc_yes,2,0,1);
	place_component(carnac_gc_no,3,0,1);

	place_buttons(1);

	getContentPane().add(soft_panel);
	pack();
	setVisible(true);


    }


    
    //Build the dialog for rnaz configuration
    public void build_rnaz(){

	//Get the content of rnaz configuration file
	current_conf_file = main_window.get_rnaz_conf();
	content = main_window.get_conf_content(current_conf_file);

	//Get future location of the new rnaz configuration file
	new_conf_file = main_window.get_temp_rnaz_conf();

	//Get the default configuration file
	default_conf_file = main_window.get_default_rnaz_conf();

	//Regular expression for the probability cutoff
	rnaz_proba_regexp = "(-p[ \\t]*)(\\d?\\.?\\d+)";
	//Get the current probability cutoff value
	rnaz_proba_value = get_parameter_value(content,
					       rnaz_proba_regexp);

	//Set the array of parameters
	parameters = new String[1];
	parameters[0] = rnaz_proba_regexp+":"+rnaz_proba_value;

	//Probability cutoff parameter
	JLabel rnaz_proba_label = new JLabel("Probability cutoff (0<->1)");
	place_component(rnaz_proba_label,0,0,2);

	rnaz_proba_text = new JTextField();
	rnaz_proba_text.setPreferredSize(small_dimension);
	rnaz_proba_text.setMinimumSize(small_dimension);
	rnaz_proba_text.setText(rnaz_proba_value);
	rnaz_proba_text.addFocusListener(new Focus_listener(this,0,false));
	place_component(rnaz_proba_text,2,0,3);

	place_buttons(1);

	
	getContentPane().add(soft_panel);
	pack();
	setVisible(true);

    }

    //Place a component on the panel according to constraints
    public void place_component(JComponent component,
				int gridx, 
				int gridy,
				int gridwidth){


	constraints.gridx = gridx;
	constraints.gridy = gridy;
	constraints.gridwidth = gridwidth;

	soft_panel.add(component, constraints);



    }
				



    //Place buttons on panel according to gridy constraint
    public void place_buttons(int gridy){

	//Buttons
	button_save = new JButton("Save");
	button_save.addActionListener(this);
	place_component(button_save,0,gridy,1);

	button_cancel = new JButton("Cancel");
	button_cancel.addActionListener(this);
	place_component(button_cancel,1,gridy,1);

	button_default = new JButton("Reset to default");
	button_default.addActionListener(this);
	place_component(button_default,3,gridy,1);

	button_advanced = new JButton("Advanced");
	button_advanced.addActionListener(this);
	place_component(button_advanced,4,gridy,1);

	
    }





    //Get a regular expression that matches a parameter line (e.i param value)
    //and return the parameter value that is contained inside the string content
    //if no pattern was found returns empty string. 
    public String get_parameter_value(String content, 
				      String regexp){

	//Returned string
	String value = "";

	//Compilation of the regular expression
	Pattern pattern = Pattern.compile(regexp);

	//Find the occurence
	Matcher matcher = pattern.matcher(content);

	if(matcher.find()){
	    //Get the value : the group at index 2 
	    value = matcher.group(2);

	}
	

	return value;


    }


    //Function that will replace the old parameters by the new one in the
    //content of the configuration file
    public void replace_parameter(String param){

	//Get the regular expression that matches the parameter and its value
	//->Split param according to ":" , first the regexp then the value
	String[] split_param = param.split(":");
	String regexp = split_param[0];
	String value = split_param[1];

	//Regexp compilation
	Pattern pattern = Pattern.compile(regexp);
    
	//Replace the occurence
	Matcher matcher = pattern.matcher(content);

	if(matcher.find()){
	    String new_content = matcher.replaceAll("$1"+value);
	    content = new_content;
	}

    }


    


    //Function that will returned the new conf file
    public String get_new_conf_file(){

	return new_conf_file;
    }



    public void actionPerformed(ActionEvent e){

	//Cancel button
	if(e.getSource() == button_cancel){

	    new_conf_file = current_conf_file;
	    setVisible(false);

	}
	else{
	    //Save button
	    if(e.getSource() == button_save){
		

		//Check parameters validity
		boolean is_valid = check_parameters_validity();

		if(is_valid){
		    
		    //Get all parameters and replace them inside the content
		    //before saving the new configuration file
		    for(int i = 0; i < parameters.length; i++){
			replace_parameter(parameters[i]);
			
		    }

		    //Write new content inside temporary file
		    main_window.write_conf_content(content,new_conf_file);

		    setVisible(false);
		
		}


	    }
	    else{

		//Default button
		if(e.getSource() == button_default){
	    
		    //Reset values to default
		    reset_to_default();

		}
		else{
		    //Advanced button
		    if(e.getSource() == button_advanced){
			
			//Show advanced window
			Advanced_software_window advanced_window =
			    new Advanced_software_window(main_window,
							 main_conf_window,
							 this,
							 current_conf_file,
							 default_conf_file,
							 new_conf_file);
			
			setVisible(false);
			advanced_window.build();
			

		    }
	    

		    
		    //Else change of parameter value
		    else{
			
			//Case JComboBox
			if((e.getSource() instanceof JComboBox)){
			    
			    //Get new value 
			    JComboBox combo = (JComboBox) e.getSource();
			    String new_value = (String)combo.getSelectedItem();
			    
			    String[] split_values = new_value.split(";");
			    String[] indexes_split = 
				(e.getActionCommand()).split(":");
			    
			    for(int i = 0; i < split_values.length; i++){
				String current_value = split_values[i];
				int current_index = 
				    Integer.parseInt(indexes_split[i]);
				
				//Replace value
				update_parameters(current_index, current_value);
				
				
			    }
			    
			}
			else{
			    //Radio buttons action command are made of index and value
			    //separated by the ":" character
			    String[] split_action = 
				(e.getActionCommand()).split(":");
			    int index_param = Integer.parseInt(split_action[0]);
			    String new_value = split_action[1];
			    
			    //Replace value
			    update_parameters(index_param, new_value);
			}
		    }	

		}
	    }
	}
    }


    //Function that will check parameters validity
    public boolean check_parameters_validity(){


	//Boolean that represents validity parameters
	boolean is_valid = true;

	//Error message
	String error_message;

	//Error title
	String error_title;
	
	//Get current software
	String current_software = main_conf_window.get_selected_soft();


	//Masking redundancy case
	if(current_software.equals("clean up redundancy")){

	    //Length value
	    masking_redundancy_length_value =
		masking_redundancy_length_text.getText();
	    is_valid = is_valid_parameter(masking_redundancy_length_value,
					  "\\d+");
	    if(!is_valid){
		error_message = "Wrong length";
		error_title = "ERROR : Length error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Integer.parseInt(masking_redundancy_length_value) <= 0){
		error_message = "The length must be higher than 0";
		error_title = "ERROR : Length error...";
		show_error(error_message, error_title);
		return false;

	    }

	    //Identity value
	    masking_redundancy_id_value =
		masking_redundancy_id_text.getText();
	    is_valid = is_valid_parameter(masking_redundancy_id_value,
					  "\\d+(\\.\\d+)?");
	    if(!is_valid){
		error_message = "Wrong identity";
		error_title = "ERROR : Identity error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if((Float.parseFloat(masking_redundancy_id_value) <= 0) ||
	       (Float.parseFloat(masking_redundancy_id_value) > 100)){
		error_message = "The identity must be between 1 and 100";
		error_title = "ERROR : Identity error...";
		show_error(error_message, error_title);
		return false;

	    }

	}

	//Yass case
	if(current_software.equals("yass")){

	    //Check Evalue
	    yass_evalue_value = yass_evalue_text.getText();
	    is_valid = is_valid_parameter(yass_evalue_value,
					  "\\d*\\.?\\d+(e\\-?\\d+)?");
	    if(!is_valid){
		error_message = "Wrong E-value";
		error_title = "ERROR : E-value error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Float.parseFloat(yass_evalue_value) < 0){
		error_message = "The E-value must be higher than 0";
		error_title = "ERROR : E-value error...";
		show_error(error_message, error_title);
		return false;

	    }
	    

	    //Costs
	    yass_costs_value = yass_costs_text.getText();
	    is_valid = is_valid_parameter(yass_costs_value,
					  "\\-?\\d+,\\-?\\d+,\\-?\\d+,\\-?\\d+");
	    if(!is_valid){
		error_message = 
		    "Wrong match/transition/transverion/other costs";
		error_title = "ERROR : Costs error...";
		show_error(error_message, error_title);
		return false;
	    }
	    
	    //Gap costs
	    yass_gap_costs_value = yass_gap_text.getText();
	    is_valid = is_valid_parameter(yass_gap_costs_value,
					  "\\-?\\d+,\\-?\\d+");
	    if(!is_valid){
		error_message = 
		    "Wrong gap opening and extension costs";
		error_title = "ERROR : Gap costs error...";
		show_error(error_message, error_title);
		return false;
	    }


	    //Seed
	    yass_seed_value = yass_seed_text.getText();
	    is_valid = is_valid_parameter(yass_seed_value,
					  "^[^,][#\\-@,]+");
	    if(!is_valid){
		error_message = 
		    "Wrong seed(s) : be sure to use valid characters (\"#\",\"@\",\"_\" and \",\" to separate seeds)";
		error_title = "ERROR : Seed error...";
		show_error(error_message, error_title);
		return false;
	    }
	}


	//Blast case
	if(current_software.equals("blast")){

	    //Check Evalue
	    blast_evalue_value = blast_evalue_text.getText();
	    is_valid = is_valid_parameter(blast_evalue_value,
					  "\\d*\\.?\\d+(e\\-?\\d+)?");
	    if(!is_valid){
		error_message = "Wrong E-value";
		error_title = "ERROR: E-value error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Float.parseFloat(blast_evalue_value) < 0){
		error_message = "The E-value must be higher than 0";
		error_title = "ERROR: E-value error...";
		show_error(error_message, error_title);
		return false;

	    }


	    //Word size
	    blast_word_value = blast_word_text.getText();
	    is_valid = is_valid_parameter(blast_word_value,"[\\d]+");
	    if(!is_valid){
		error_message = "Wrong word size";
		error_title = "ERROR: Word size error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if((Integer.parseInt(blast_word_value) < 7) ||
	       (Integer.parseInt(blast_word_value) > 30)){
		error_message = "The word size must be between 7 and 30";
		error_title = "ERROR : Word size error...";
		show_error(error_message, error_title);
		return false;

	    }


	}

	//CG-seq case
	if(current_software.equals("conservedCG-seq")){

	    //Minimum identity
	    cg_seq_min_id_value = cg_seq_min_id_text.getText();
	    is_valid = is_valid_parameter(cg_seq_min_id_value,"\\d+");
	    if(!is_valid){
		error_message = "Wrong minimum identity threshold";
		error_title = "ERROR: Minimum identity error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if((Integer.parseInt(cg_seq_min_id_value) < 0) ||
	       (Integer.parseInt(cg_seq_min_id_value) > 100)){
		error_message = "The minimum identity must be between 0 and 100";
		error_title = "ERROR: Minimum identity error...";
		show_error(error_message, error_title);
		return false;
		
	    }

	    //Maximum identity
	    cg_seq_max_id_value = cg_seq_max_id_text.getText();
	    is_valid = is_valid_parameter(cg_seq_max_id_value,"\\d+");
	    if(!is_valid){
		error_message = "Wrong maximum identity threshold";
		error_title = "ERROR : Maximum identity error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if((Integer.parseInt(cg_seq_max_id_value) < 0) ||
	       (Integer.parseInt(cg_seq_max_id_value) > 100)){
		error_message = "The maximum identity must be bewteen 0 and 100";
		error_title = "ERROR : Maximum identity error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Integer.parseInt(cg_seq_max_id_value) < 
	       Integer.parseInt(cg_seq_min_id_value)){
		error_message = "The maximum identity must be higher than the minimum identity";
		error_title = "ERROR: Identity threshold error...";
		show_error(error_message, error_title);
		return false;
	    }

	    //Minimum length
	    cg_seq_min_length_value = cg_seq_min_length_text.getText();
	    is_valid = is_valid_parameter(cg_seq_min_length_value,"\\d+");
	    if(!is_valid){
		error_message = "Wrong minimum length";
		error_title = "ERROR: Minimum length error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Integer.parseInt(cg_seq_min_id_value) < 0){
		error_message = "The minimum length must be higher than 0";
		error_title = "ERROR : Minimum length error...";
		show_error(error_message, error_title);
		return false;
		
	    }

	    //Maximum length
	    cg_seq_max_length_value = cg_seq_max_length_text.getText();
	    is_valid = is_valid_parameter(cg_seq_max_length_value,"\\d+");
	    if(!is_valid){
		error_message = "Wrong maximum length";
		error_title = "ERROR : Maximum length error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Integer.parseInt(cg_seq_max_length_value) < 0){
		error_message = "The maximum length must be higher than 0";
		error_title = "ERROR : Maximum length error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Integer.parseInt(cg_seq_max_length_value) <
	       Integer.parseInt(cg_seq_min_length_value)){
		error_message = "The maximum length must be higher than the minimum length";
		error_title = "ERROR:Length error...";
		show_error(error_message, error_title);
		return false;
	    }

	    //Number of sequences
	    cg_seq_nb_seqs_value = cg_seq_nb_seqs_text.getText();
	    is_valid = is_valid_parameter(cg_seq_nb_seqs_value,"\\d+");
	    if(!is_valid){
		error_message = "Wrong number of sequences";
		error_title = "ERROR : Nmber of sequences error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if(Integer.parseInt(cg_seq_nb_seqs_value) < 0){
		error_message = "The number of sequences must be higher than 0";
		error_title = "ERROR : Number of sequences error...";
		show_error(error_message, error_title);
		return false;
	    }
	    
	}

	//Carnac case
	if(current_software.equals("carnac")){

	    //There are no parameter to check since the only parameter
	    //is made of radio buttons with valid values
	    return true;
	    
	}

	//Rnaz case
	if(current_software.equals("rnaz")){


	    //Probability cutoff
	    rnaz_proba_value = rnaz_proba_text.getText();
	    is_valid = is_valid_parameter(rnaz_proba_value,"\\d?\\.?\\d+");
	    if(!is_valid){
		error_message = "Wrong probability cutoff";
		error_title = "ERROR : Probability cutoff error...";
		show_error(error_message, error_title);
		return false;
	    }
	    if((Float.parseFloat(rnaz_proba_value) < 0) ||
	       (Float.parseFloat(rnaz_proba_value) > 1)){
		error_message = "The probability cutoff must be between 0 and 1";
		error_title = "ERROR : Probability cutoff error...";
		show_error(error_message, error_title);
		return false;
	    }

	}


	return true;

    }


    //Show an error message dialog
    public void show_error(String error_message, String error_title){

	JOptionPane error_pane = new JOptionPane();
	error_pane.showMessageDialog(this,
				     error_message,
				     error_title,
				     JOptionPane.ERROR_MESSAGE);


    }


    

    //Check if a parameter respects a regular expression
    public boolean is_valid_parameter(String param, String regexp){


	//Regexp compilation
	Pattern pattern = Pattern.compile(regexp);
    
	//Replace the occurence
	Matcher matcher = pattern.matcher(param);

	if(matcher.matches()){
	    return true;
	}
	else{
	    return false;
	}

    }


    



    //Function that will update the parameters array at the index chosen
    //with the new value
    public void update_parameters(int index, String value){


	//Replace value at the correct index
	String[] params = parameters[index].split(":");
	parameters[index] = params[0]+":"+value;



    }



    //Function that will reset to default values
    public void reset_to_default(){

	//Get current software
	String current_software = main_conf_window.get_selected_soft();

	//Get the content of the default masking configuration file
	content = main_window.get_conf_content(default_conf_file);



	//Masking redundancy panel case
	if(current_software.equals("clean up redundancy")){


	    //Get the default length 
	    masking_redundancy_length_value = 
		get_parameter_value(content,
				    masking_redundancy_length_regexp);

	    //Get the default identity 
	    masking_redundancy_id_value = 
		get_parameter_value(content,
				    masking_redundancy_id_regexp);

	    //Update parameters
	    update_parameters(0,masking_redundancy_length_value);
	    update_parameters(1,masking_redundancy_id_value);
	 
	    masking_redundancy_length_text.setText(masking_redundancy_length_value);
	    masking_redundancy_id_text.setText(masking_redundancy_id_value);

   
	}


	//Yass panel case
	if(current_software.equals("yass")){

	    //Get each values
	    yass_strand_value = get_parameter_value(content,
						    yass_strand_regexp);
	    yass_evalue_value = get_parameter_value(content,
						    yass_evalue_regexp);
	    yass_costs_value = get_parameter_value(content,
						   yass_costs_regexp);	    
	    yass_gap_costs_value = get_parameter_value(content,
						       yass_gap_costs_regexp);
	    yass_seed_value = get_parameter_value(content,
						  yass_seed_regexp);
	    yass_hit_criterion_value = 
		get_parameter_value(content,
				    yass_hit_criterion_regexp);

	    yass_low_complexity_value = 
		get_parameter_value(content,
				    yass_low_complexity_regexp);

	    //Update parameters
	    update_parameters(0,yass_strand_value);
	    update_parameters(1,yass_evalue_value);
	    update_parameters(2,yass_costs_value);
	    update_parameters(3,yass_gap_costs_value);
	    update_parameters(4,"\""+yass_seed_value+"\"");
	    update_parameters(5,yass_hit_criterion_value);
	    update_parameters(6,yass_low_complexity_value);

	    if(yass_strand_value.equals("2")){
		yass_radio_strand_both.setSelected(true);
	    } 
	    else{
		if(yass_strand_value.equals("0")){
		    yass_radio_strand_forward.setSelected(true);
		} 
		else{
		    yass_radio_strand_reverse.setSelected(true);
		}
	    }
	    yass_evalue_text.setText(yass_evalue_value);
	    yass_costs_text.setText(yass_costs_value);
	    yass_gap_text.setText(yass_gap_costs_value);
	    yass_seed_text.setText(yass_seed_value);
	    if(yass_hit_criterion_value.equals("1")){
		yass_radio_hit_simple.setSelected(true);
	    }
	    else{
		yass_radio_hit_double.setSelected(true);
	    }
	    if(yass_low_complexity_value.equals("0")){
		yass_radio_filter_no.setSelected(true);
	    }
	    else{
		yass_radio_filter_yes.setSelected(true);	
	    }


	}

	//Blast panel case
	if(current_software.equals("blast")){
    
	    //Get each values
	    blast_strand_value = get_parameter_value(content,
						     blast_strand_regexp);
	    blast_evalue_value = get_parameter_value(content,
						     blast_evalue_regexp);
	    blast_word_value = get_parameter_value(content,
						   blast_word_regexp);
	    blast_match_value = get_parameter_value(content,
						    blast_match_regexp);    
	    blast_mismatch_value = get_parameter_value(content,
						       blast_mismatch_regexp);
	    blast_gap_open_value = get_parameter_value(content,
						       blast_gap_open_regexp);
	    blast_gap_ext_value = get_parameter_value(content,
						      blast_gap_ext_regexp);
	    String blast_all_costs_value = blast_match_value+
		"; "+blast_mismatch_value+
		"; "+blast_gap_open_value+
		"; "+blast_gap_ext_value;

	    blast_low_complexity_value = 
		get_parameter_value(content,
				    blast_low_complexity_regexp);


	    //update parameters
	    update_parameters(0,blast_strand_value);
	    update_parameters(1,blast_evalue_value);
	    update_parameters(2,blast_word_value);
	    update_parameters(3,blast_match_value);
	    update_parameters(4,blast_mismatch_value);
	    update_parameters(5,blast_gap_open_value);
	    update_parameters(6,blast_gap_ext_value);
	    update_parameters(7,blast_low_complexity_value);

	    if(blast_strand_value.equals("3")){
		blast_radio_strand_both.setSelected(true);
	    }
	    else{
		if(blast_strand_value.equals("1")){
		    blast_radio_strand_forward.setSelected(true);
		}
		else{
		    blast_radio_strand_reverse.setSelected(true); 
		}

	    }
	    blast_evalue_text.setText(blast_evalue_value);
	    blast_word_text.setText(blast_word_value);
	    blast_costs_combo_box.setSelectedItem(blast_all_costs_value);
	    if(blast_low_complexity_value.equals("T")){
		blast_radio_filter_yes.setSelected(true);
	    }
	    else{
		blast_radio_filter_no.setSelected(true);	
	    }

	}

	
	//Cg_Seq panel case
	if(current_software.equals("CG-seq")){


	    //Get each values
	    cg_seq_min_id_value = get_parameter_value(content,
						      cg_seq_min_id_regexp);
	    cg_seq_max_id_value = get_parameter_value(content,
						      cg_seq_max_id_regexp);
	    cg_seq_mode_value = get_parameter_value(content,
						    cg_seq_mode_regexp);
	    cg_seq_max_length_value = get_parameter_value(content,
							  cg_seq_max_length_regexp);
	    cg_seq_min_length_value = get_parameter_value(content,
							  cg_seq_min_length_regexp);
	    cg_seq_nb_seqs_value = get_parameter_value(content,
						       cg_seq_nb_seqs_regexp);

	    
	    //update parameters
	    update_parameters(0,cg_seq_min_id_value);
	    update_parameters(1,cg_seq_max_id_value);
	    update_parameters(2,cg_seq_min_length_value);
	    update_parameters(3,cg_seq_max_length_value);
	    update_parameters(4,cg_seq_mode_value);
	    update_parameters(5,cg_seq_nb_seqs_value);

	    cg_seq_min_id_text.setText(cg_seq_min_id_value);
	    cg_seq_max_id_text.setText(cg_seq_max_id_value);
	    cg_seq_min_length_text.setText(cg_seq_min_length_value);
	    cg_seq_max_length_text.setText(cg_seq_max_length_value);
	    if(cg_seq_mode_value.equals("0")){
		cg_seq_mode_star.setSelected(true);
	    }
	    else{
		if(cg_seq_mode_value.equals("1")){
		    cg_seq_mode_naive.setSelected(true);
		}
		else{
		   cg_seq_mode_clique.setSelected(true); 
		}
	    }
	    cg_seq_nb_seqs_text.setText(cg_seq_nb_seqs_value);
	    
	}


	//Carnac panel case
	if(current_software.equals("carnac")){

	    //Get value
	    carnac_gc_value_used = get_parameter_value(content,
						       carnac_gc_regexp_used);

	    //Check whether that option is used or not
	    boolean carnac_gc_used = carnac_gc_value_used.equals("-A");

	    //Update parameter
	    if(carnac_gc_used){
		update_parameters(0,"-A\n");
		carnac_gc_yes.setSelected(true);
	    }
	    else{
		carnac_gc_no.setSelected(true);
		update_parameters(0,"#-A\n");
	    }



	}


	//RNAz panel case
	if(current_software.equals("rnaz")){

	    //Get value
	    rnaz_proba_value = get_parameter_value(content,
						   rnaz_proba_regexp);

	    //Update parameter
	    update_parameters(0,rnaz_proba_value);
	    rnaz_proba_text.setText(rnaz_proba_value);



	}

    }



    //Set new configuration file to use
    public void set_new_conf_file(String new_conf_file){
	this.new_conf_file = new_conf_file;
    }



}
