///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import javax.swing.JOptionPane;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Desktop;
import java.net.URI;
import java.net.URISyntaxException;
import java.io.IOException;
import java.io.File;
import java.util.Arrays;

//Cancel buttons listener
public class Help_button_listener implements ActionListener {

    //Main window
    private Main_window main_window = null;

    //Anchor in the help file
    private String anchor;


    //Constructor
    public Help_button_listener(Main_window main_window, String anchor){

	super();
	this.main_window = main_window;
	this.anchor = anchor;

    }

    public void actionPerformed(ActionEvent e){

	//Test if desktop object is supported
	if(Desktop.isDesktopSupported()){

	    Desktop desktop = Desktop.getDesktop();

	    //Test is browser action is supported
	    if(desktop.isSupported(Desktop.Action.BROWSE)) {

		try{

		    String help_location = main_window.get_help_file();

		    File help_file = new File(help_location);
		    String help_file_real_location = 
			help_file.getAbsolutePath();		    

		    File test_file = new File(help_file_real_location);

		    if(test_file.exists()){

			help_file_real_location = 
			    help_file_real_location.replaceAll("\\\\","/");
			help_file_real_location = 
			    help_file_real_location.replaceAll(" ","%20");
			help_file_real_location += "#"+anchor;
			URI help_page = 
			    new URI("file:///"+help_file_real_location);
			try{
			    desktop.browse(help_page);
			}
			catch(IOException ioe){		    
			    look_for_browser(help_page);
			}
		    }
		    else{
			help_file_error(help_file_real_location);
		    }

		}
		catch(IOException ioe){		    
		    ioe.printStackTrace();
		}
		catch(URISyntaxException urie){
		    urie.printStackTrace();
		}

	    }
	    else{
		browser_error();
	    }

	}
	else{
	    browser_error();
	}

    }




    //Show an error panel when the browser cannot be launched to
    //display results
    public void browser_error(){

	JOptionPane error_pane = new JOptionPane();

	String error_message = "The browser cannot be launched...";

	String error_title = "ERROR : Cannot launch the browser";

	error_pane.showMessageDialog(main_window,
				     error_message,
				     error_title,
				     JOptionPane.ERROR_MESSAGE);


    }


    //Show an error panel when the help file doesn't exist
    public void help_file_error(String help_file){

	JOptionPane error_pane = new JOptionPane();

	String error_message = 
	    "The help file "+help_file+" doesn't exist...";

	String error_title = "ERROR : Cannot find help file";

	error_pane.showMessageDialog(main_window,
				     error_message,
				     error_title,
				     JOptionPane.ERROR_MESSAGE);


    }

    private static void look_for_browser(URI uri) throws IOException {
	String[] commands = {
	    System.getenv("BROWSER"),
	    "xdg-open",
	    "kfmclient exec",
	    "exo-open",	
	    "gnome-open",
	    "firefox",
	    "konqueror",
	    "netscape"
	};
	
	for (String cmd : commands) {
	    if (cmd !=null) {
		try {
		    browse(cmd, uri);
		    return; 
		} catch (IOException e) {
		    System.err.println(cmd + " : " + e.getMessage());
		}
	    }
	}
	throw new IOException("no browser found");	
    }
    
    private static void browse(String command, URI uri) throws IOException {
// 	String[] args = command.split(" +");
// 	args = Arrays.copyOf(args, args.length+1);
// 	args[args.length-1] = uri.toString();
	
	String args = command + " " + uri.toString();
	final Runtime runtime = Runtime.getRuntime();
	final Process process = runtime.exec(args);
	process.getOutputStream().close();
	process.getInputStream().close();
	process.getErrorStream().close();
    }
}
