///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import javax.swing.UIManager;
import javax.swing.SwingConstants;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JFileChooser;
import javax.swing.JSeparator;
import javax.swing.ImageIcon;
import javax.swing.SwingConstants;
import javax.swing.text.Document;
import javax.swing.text.BadLocationException;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Insets;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GraphicsEnvironment;
import java.awt.Cursor;
import java.net.URL;
import java.lang.ClassLoader;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.lang.Exception;

//Main window class
public class Main_window extends JFrame implements MouseListener {

    //Panels
    private JPanel main_panel = null;
    private JPanel load_panel = null;
    private JPanel soft_panel = null;
      

    //Layout and constraints
    private GridBagLayout main_layout = null;
    private GridBagConstraints main_constraints = null;
    private GridBagLayout load_layout = null;
    private GridBagConstraints load_constraints = null;
    private GridBagLayout soft_layout = null;
    private GridBagConstraints soft_constraints = null;

    //GridBagConstraints constants
    private int anchor_center = GridBagConstraints.CENTER;
    private int anchor_line_start = GridBagConstraints.LINE_START;
    private int anchor_line_end = GridBagConstraints.LINE_END;
    private int anchor_page_start = GridBagConstraints.PAGE_START;
    private int anchor_page_end = GridBagConstraints.PAGE_END;
    private int fill_none = GridBagConstraints.NONE;
    private int fill_horizontal = GridBagConstraints.HORIZONTAL;
    private int fill_vertical = GridBagConstraints.VERTICAL;
    
    //Insets
    private Insets inset_zero_space = null;
    private Insets inset_soft = null;
    private Insets inset_conf_button = null;
    private Insets inset_vertical_title = null;
    private Insets inset_main = null;
    private Insets inset_checkbox = null;
    private Insets inset_vspace = null;

    //Images
    private ImageIcon cancel_image = null;
    private ImageIcon open_image = null;
    private ImageIcon help_image = null;
    private ImageIcon help_small_image = null;
    private ImageIcon arrow_image = null;

    //Font
    private Font font_title = null;
    private Font font_bold = null;
    private Font font_text = null;
    private Font font_button_bold = null;
    private Font font_plain = null;
   
    //Dimensions
    private Dimension button_dimension = null;
    private Dimension textfield_dimension = null;
    private Dimension textarea_dimension = null;
    private Dimension conf_button_dimension = null;
    private Dimension run_dimension = null;
    private Dimension help_dimension = null;
    private Dimension help_soft_dimension = null;

    //Textfields and TextAreas
    private JTextField ref_text_field = null;
    private JTextArea other_text_area = null;
    private JTextField outdir_text_field = null;

    //ScrollPane
    private JScrollPane other_scrollpane = null;

    //Labels
    private JLabel label_title = null;
    private JLabel label_intro_file = null;
    private JLabel label_ref_file = null;
    private JLabel label_other_dir = null;
    private JLabel label_outdir = null;
    private JLabel label_intro_pipeline = null;
    private JLabel label_mask = null;
    private JLabel label_alignment = null;
    private JLabel label_conserved = null;
    private JLabel label_structure = null;

    //Buttons
    private JButton ref_open_button = null;
    private JButton ref_cancel_button = null;
    private JButton ref_help_button = null;
    private JButton other_open_button = null;
    private JButton other_cancel_button = null;
    private JButton other_help_button = null;
    private JButton outdir_open_button = null;
    private JButton outdir_cancel_button = null;
    private JButton outdir_help_button = null;
    private JButton conf_seq_button = null;
    private JButton conf_seq_help_button = null;
    private JButton conf_soft_button = null;
    private JButton conf_soft_help_button = null;
    private JButton run_button = null;
    private JButton help_button = null;
    private JButton button_help_mask_cds = null;
    private JButton button_help_mask_rna = null;
    private JButton button_help_mask_redundancy = null;
    private JButton button_help_yass = null;
    private JButton button_help_blast = null;
    private JButton button_help_cg_seq = null;
    private JButton button_help_carnac = null;
    private JButton button_help_rnaz = null;
    private JButton button_arrow1 = null;
    private JButton button_arrow2 = null;
    private JButton button_arrow3 = null;

    //Checkboxes
    private JCheckBox mask_cds_checkbox = null;
    private JCheckBox mask_rna_checkbox = null;
    private JCheckBox mask_redundancy_checkbox = null;
    private JCheckBox yass_checkbox = null;
    private JCheckBox blast_checkbox = null;
    private JCheckBox carnac_checkbox = null;
    private JCheckBox rnaz_checkbox = null;

    //Separators
    private JSeparator separator_hor1 = null;
    private JSeparator separator_hor2 = null;
    private JSeparator separator_hor3 = null;
    private JSeparator separator_hor4 = null;
    private JSeparator separator_hor5 = null;
    private JSeparator separator_hor6 = null;
    private JSeparator separator_vert1 = null;
    private JSeparator separator_vert2 = null;
    private JSeparator separator_vert3 = null;

    //Listeners
    private Open_button_listener ref_open_listener = null;
    private Open_button_listener other_open_listener = null;
    private Open_button_listener outdir_open_listener = null;
    private Cancel_button_listener ref_cancel_listener = null;
    private Cancel_button_listener other_cancel_listener = null;
    private Cancel_button_listener outdir_cancel_listener = null;
    private Run_button_listener run_listener = null;
    private Configure_sequences_button_listener conf_seq_listener = null;
    private Configure_software_button_listener conf_soft_listener = null;
    private Help_button_listener ref_help_listener = null;
    private Help_button_listener other_help_listener = null;
    private Help_button_listener outdir_help_listener = null;
    private Help_button_listener conf_seq_help_listener = null;
    private Help_button_listener mask_regions_help_listener = null;
    private Help_button_listener mask_redundancy_help_listener = null;
    private Help_button_listener yass_help_listener = null;
    private Help_button_listener blast_help_listener = null;
    private Help_button_listener cg_seq_help_listener = null;
    private Help_button_listener carnac_help_listener = null;
    private Help_button_listener rnaz_help_listener = null;
    private Help_button_listener conf_soft_help_listener = null;
    private Help_button_listener general_help_listener = null;


    //Configuration files values

    //Path to initial configuration directory
    private String conf_initial_dir;

    //Path to temporary configuration directory
    private String conf_temp_dir;

    //Name of configuration file suffix
    private String conf_suffix;

    //Configuration files for the masking regions step (default, temp, current)
    private String default_masking_regions_conf;
    private String temp_masking_regions_conf;
    private String masking_regions_conf_to_use;

    //Configuration files for the masking of redundancy reginons step (default, temp, current)
    private String default_masking_redundancy_conf;
    private String temp_masking_redundancy_conf;
    private String masking_redundancy_conf_to_use;

    //Configuration files for Yass (default, temp, current)
    private String default_yass_conf;
    private String temp_yass_conf;
    private String yass_conf_to_use;

    //Configuration files for Blast (default, temp, current)
    private String default_blast_conf;
    private String temp_blast_conf;
    private String blast_conf_to_use;

    //Configuration files for Cg_Seq (default, temp, current)
    private String default_cg_seq_conf;
    private String temp_cg_seq_conf;
    private String cg_seq_conf_to_use;

    //Configuration files for Carnac (default, temp, current)
    private String default_carnac_conf;
    private String temp_carnac_conf;
    private String carnac_conf_to_use;

    //Configuration files for RNAz (default, temp, current)
    private String default_rnaz_conf;
    private String temp_rnaz_conf;
    private String rnaz_conf_to_use;

    //Script to launch the perl script
    private String perl_script;

    //Default probability
    private String default_proba = "0.8";

    //Array of species probability values
    private String[] seq_proba;

    //Hashtable that links the sequence to its probability
    private Hashtable<String,String> hash_proba;

    //Help file
    private String help_file; 


    //Constructor
    public Main_window(){

	super();

	//Images
	URL open_image_url = getClass().getResource("images/folder_open.png");
	URL cancel_image_url = getClass().getResource("images/gtk-cancel.png");
	URL help_image_url = getClass().getResource("images/help.png");
	URL help_small_image_url = getClass().getResource("images/help_small.png");
	URL arrow_image_url = getClass().getResource("images/go-next.png");

	open_image = new ImageIcon(open_image_url);
	cancel_image = new ImageIcon(cancel_image_url);
	help_image = new ImageIcon(help_image_url);
	help_small_image = new ImageIcon(help_small_image_url);
	arrow_image = new ImageIcon(arrow_image_url);

	//Get jar real location
	String jar = getClass().getProtectionDomain().getCodeSource().getLocation().getPath();
	jar = jar.replaceAll("/","\\"+File.separator);

	//Windows case : delete first "\" character
	if(jar.startsWith("\\")){
	    jar = jar.replaceFirst("\\"+File.separator,"");
	}
	    
	String jar_path = jar.replaceAll("CG-seq.jar","");
	jar_path = jar_path.replaceAll("%20"," ");


	//Configuration part
	this.conf_initial_dir = jar_path+"cfg/";
	this.conf_temp_dir = conf_initial_dir+"temp_cfg/";
	this.conf_suffix = ".conf";

	//Mask coding part
	this.default_masking_regions_conf = conf_initial_dir+"mask_regions"+conf_suffix;
	this.temp_masking_regions_conf = conf_temp_dir+"mask_regions"+conf_suffix;
	this.masking_regions_conf_to_use = default_masking_regions_conf;

	//Mask redundancy part
	this.default_masking_redundancy_conf = conf_initial_dir+"mask_redundancy"+conf_suffix;
	this.temp_masking_redundancy_conf = conf_temp_dir+"mask_redundancy"+conf_suffix;
	this.masking_redundancy_conf_to_use = default_masking_redundancy_conf;

	//Yass part
	this.default_yass_conf = conf_initial_dir+"yass"+conf_suffix;
	this.temp_yass_conf = conf_temp_dir+"yass"+conf_suffix;
	this.yass_conf_to_use = default_yass_conf;

	//Blast part
	this.default_blast_conf = conf_initial_dir+"blast"+conf_suffix;
	this.temp_blast_conf = conf_temp_dir+"blast"+conf_suffix;
	this.blast_conf_to_use = default_blast_conf;

	//Cg_Seq part
	this.default_cg_seq_conf = 
	    conf_initial_dir+"cgseqcore"+conf_suffix;
	this.temp_cg_seq_conf = conf_temp_dir+"cgseqcore"+conf_suffix;
	this.cg_seq_conf_to_use = default_cg_seq_conf;

	//Carnac part
	this.default_carnac_conf = conf_initial_dir+"carnac"+conf_suffix;
	this.temp_carnac_conf = conf_temp_dir+"carnac"+conf_suffix;
	this.carnac_conf_to_use = default_carnac_conf;

	//RNAz part
	this.default_rnaz_conf = conf_initial_dir+"rnaz"+conf_suffix;
	this.temp_rnaz_conf = conf_temp_dir+"rnaz"+conf_suffix;
	this.rnaz_conf_to_use = default_rnaz_conf;

	//Perl script 
	this.perl_script = jar_path+"CG-seq.pl";

	File temp_dir = new File(conf_temp_dir);
	if(!temp_dir.exists()){
	    temp_dir.mkdir();  
	}

	this.hash_proba = new Hashtable<String,String>();

	//Help file
	this.help_file = jar_path+"doc/help.html";

	build();


    }



    ////
    //  Build functions
    ////


    //Build fonction
    private void build(){

	Boolean nimbus = false;

	for (UIManager.LookAndFeelInfo laf :
		 UIManager.getInstalledLookAndFeels() ){
	    //Nimbus Look & Feel is available
	    if ("Nimbus".equals(laf.getName())) {
		try {
		    UIManager.setLookAndFeel(laf.getClassName());
		    nimbus = true;

		} catch (Exception e) {
		    e.printStackTrace();
		}

		button_dimension = new Dimension(32,32);
		textfield_dimension = new Dimension(500,27);
		textarea_dimension = new Dimension(500,81);
		conf_button_dimension = new Dimension(185,32);
		run_dimension = new Dimension(180,40);
		help_dimension = new Dimension(100,40);
		help_soft_dimension = new Dimension(14,14);

		//Insets		
		inset_zero_space = new Insets(0,0,0,0);
		inset_conf_button  = new Insets(10,2,10,2);
		inset_vertical_title = new Insets(20,0,20,0);
		inset_main = new Insets(4,4,4,4);
		inset_soft = new Insets(10,6,10,6);
		inset_checkbox = new Insets (2,4,2,4);
		inset_vspace = new Insets(10,6,10,6);

		//Font
		font_title = new Font("DejaVu Sans",Font.BOLD,18);
		font_bold = new Font("DejaVu Sans",Font.BOLD,16);
		font_text = new Font("DejaVu Sans",Font.PLAIN,16);
		font_button_bold = new Font("DejaVu Sans",Font.BOLD,12);
		font_plain = new Font("DejaVu Sans",Font.PLAIN,12);

	    }
	}

	if(!nimbus){


	    button_dimension = new Dimension(32,32);
	    textfield_dimension = new Dimension(500,27);
	    textarea_dimension = new Dimension(500,81);
	    conf_button_dimension = new Dimension(185,32);
	    run_dimension = new Dimension(180,40);
	    help_dimension = new Dimension(100,40);
	    help_soft_dimension = new Dimension(14,14);
	    
	    //Insets		
	    inset_zero_space = new Insets(0,0,0,0);
	    inset_conf_button  = new Insets(10,2,10,2);
	    inset_vertical_title = new Insets(20,0,20,0);
	    inset_main = new Insets(4,4,4,4);
	    inset_soft = new Insets(10,6,10,6);
	    inset_checkbox = new Insets (2,4,2,4);
	    inset_vspace = new Insets(10,6,10,6);
	    
	    //Font
	    font_title = new Font("DejaVu Sans",Font.BOLD,18);
	    font_bold = new Font("DejaVu Sans",Font.BOLD,16);
	    font_text = new Font("DejaVu Sans",Font.PLAIN,16);
	    font_button_bold = new Font("DejaVu Sans",Font.BOLD,12);
	    font_plain = new Font("DejaVu Sans",Font.PLAIN,12);
	}

	this.setTitle("CG-seq");
	this.setSize(800,600);
	this.setLocationRelativeTo(null);
 	this.setResizable(false);
	this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	this.setContentPane(buildMainPanel());


   }
   


    //Build main panel
    private JPanel buildMainPanel(){


	//Main panel
	main_layout = new GridBagLayout();
	main_constraints = new GridBagConstraints();
	main_constraints.weightx = 0;
  	main_constraints.weighty = 0;

	main_panel = new JPanel(main_layout);
	main_panel.setBackground(Color.white);

	//Title
	main_constraints.insets = inset_vertical_title;
	label_title = new JLabel("CG-seq : comparative genomics");
	label_title.setFont(font_title);
	add_panel(main_panel, label_title, main_constraints,
		  0,0,1,2,anchor_center,fill_none);
	
	//Separators and load panel	
	main_constraints.insets = inset_zero_space;
	separator_hor1 = new JSeparator(JSeparator.HORIZONTAL);
	add_panel(main_panel, separator_hor1,main_constraints,
		  0,1,1,2,anchor_page_end,fill_horizontal);
	
	separator_hor2 = new JSeparator(JSeparator.HORIZONTAL);
	add_panel(main_panel, separator_hor2,main_constraints,
		  0,2,1,2,anchor_page_start,fill_horizontal);

	add_panel(main_panel, buildLoadPanel(),main_constraints,
		  0,3,1,2,anchor_line_start,fill_none);

	//Separators and soft panel

	separator_hor3 = new JSeparator(JSeparator.HORIZONTAL);
	add_panel(main_panel, separator_hor3,main_constraints,
		  0,4,1,2,anchor_page_end,fill_horizontal);
	separator_hor4 = new JSeparator(JSeparator.HORIZONTAL);
	add_panel(main_panel, separator_hor4,main_constraints,
		  0,5,1,2,anchor_page_start,fill_horizontal);
	

	
	add_panel(main_panel, buildSoftPanel(),main_constraints,
		  0,6,1,2,anchor_line_start,fill_none);
	
	
	//Separators and buttons
	separator_hor5 = new JSeparator(JSeparator.HORIZONTAL);
	add_panel(main_panel, separator_hor5,main_constraints,
		  0,7,1,2,anchor_page_end,fill_horizontal);
	separator_hor6 = new JSeparator(JSeparator.HORIZONTAL);
	add_panel(main_panel, separator_hor6,main_constraints,
		  0,8,1,2,anchor_page_start,fill_horizontal);

	main_constraints.weightx = 1;
  	main_constraints.weighty = 1;
	main_constraints.insets = inset_vspace;
	run_button = new JButton("RUN");
	run_listener = new Run_button_listener(this);
	run_button.addActionListener(run_listener);
	run_button.setPreferredSize(run_dimension);
	run_button.setMinimumSize(run_dimension);
	run_button.setForeground(Color.red);
	run_button.setToolTipText("Run pipeline");
	run_button.setFont(font_button_bold);
	add_panel(main_panel, run_button, main_constraints,
		  0,9,1,1,anchor_line_end,fill_none);

	help_button = new JButton("HELP");
	help_button.setPreferredSize(help_dimension);
	help_button.setMinimumSize(help_dimension);
	help_button.setToolTipText("Open help file in browser");
	help_button.setFont(font_button_bold);
	general_help_listener = 
	    new Help_button_listener(this,"");
	help_button.addActionListener(general_help_listener);
	add_panel(main_panel, help_button, main_constraints,
		  1,9,1,1,anchor_line_start,fill_none);
	return main_panel;
    }




    //Build Load panel
    private JPanel buildLoadPanel(){

	//Layout and constraints definitions
	load_layout = new GridBagLayout();
	load_constraints = new GridBagConstraints();
 	load_constraints.weightx = 0;
  	load_constraints.weighty = 0;


	//Load panel
	load_panel = new JPanel(load_layout);
	load_panel.setBackground(Color.white);

	
	//Files and directories part
	load_constraints.insets = inset_main;
	label_intro_file = new JLabel("LOAD DATA");
	label_intro_file.setFont(font_bold);
	add_panel(load_panel, label_intro_file, load_constraints,
		  0,3,1,15,anchor_line_start,fill_none);


	//Reference file choose
	label_ref_file = new JLabel("1.Target sequence");
	label_ref_file.setFont(font_text);
	add_panel(load_panel, label_ref_file, load_constraints,
		  0,4,1,3,anchor_line_start,fill_none);

	ref_text_field = new JTextField();
	ref_text_field.setPreferredSize(textfield_dimension);
	ref_text_field.setMinimumSize(textfield_dimension);
	add_panel(load_panel, ref_text_field, load_constraints,
		  3,4,1,9,anchor_line_start,fill_none);


	ref_open_button = new JButton(open_image);
	ref_open_listener = new Open_button_listener(ref_text_field,
						     JFileChooser.FILES_ONLY,
						     false); 
	ref_open_button.setPreferredSize(button_dimension);
	ref_open_button.setMinimumSize(button_dimension);
	ref_open_button.addActionListener(ref_open_listener);
	ref_open_button.setToolTipText("Add a target sequence");
	add_panel(load_panel, ref_open_button, load_constraints,
		  12,4,1,1,anchor_center,fill_none);


	ref_cancel_button = new JButton(cancel_image);
	ref_cancel_listener = new Cancel_button_listener(ref_text_field);
	ref_cancel_button.setPreferredSize(button_dimension);
	ref_cancel_button.setMinimumSize(button_dimension);
	ref_cancel_button.addActionListener(ref_cancel_listener);
	ref_cancel_button.setToolTipText("Remove the actual target sequence");
	add_panel(load_panel, ref_cancel_button, load_constraints,
		  13,4,1,1,anchor_center,fill_none);


	ref_help_button = new JButton(help_image);
	ref_help_button.setPreferredSize(button_dimension);
	ref_help_button.setMinimumSize(button_dimension);
	ref_help_button.setToolTipText("Help on target sequence");
	ref_help_listener = 
	    new Help_button_listener(this,"subsection_Target_sequence");
	ref_help_button.addActionListener(ref_help_listener);
	add_panel(load_panel, ref_help_button, load_constraints,
		  14,4,1,1,anchor_center,fill_none);


	//Other directories choose
	label_other_dir = new JLabel("2.Other sequences");
	label_other_dir.setFont(font_text);
	add_panel(load_panel, label_other_dir, load_constraints,
		  0,5,1,3,anchor_line_start,fill_none);

	other_text_area = new JTextArea();
	other_text_area.setRows(2);
	other_scrollpane = new JScrollPane(other_text_area,
					   JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
					   JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	other_scrollpane.setPreferredSize(textarea_dimension);
	other_scrollpane.setMinimumSize(textarea_dimension);
	add_panel(load_panel, other_scrollpane, load_constraints,
		  3,5,1,9,anchor_center,fill_none);

	other_open_button = new JButton(open_image);
	other_open_listener = new Open_button_listener(other_text_area,
						       JFileChooser.DIRECTORIES_ONLY,
						       true); 
	other_open_button.setPreferredSize(button_dimension);
	other_open_button.setMinimumSize(button_dimension);
	other_open_button.addActionListener(other_open_listener);
	other_open_button.setToolTipText("Add other sequences");
	add_panel(load_panel, other_open_button, load_constraints,
		  12,5,1,1,anchor_center,fill_none);


	other_cancel_button = new JButton(cancel_image);
	other_cancel_listener = new Cancel_button_listener(other_text_area);
	other_cancel_button.setPreferredSize(button_dimension);
	other_cancel_button.setMinimumSize(button_dimension);
	other_cancel_button.addActionListener(other_cancel_listener);
	other_cancel_button.setToolTipText("Remove actual other sequences");
	add_panel(load_panel, other_cancel_button, load_constraints,
		  13,5,1,1,anchor_center,fill_none);


	other_help_button = new JButton(help_image);
	other_help_button.setPreferredSize(button_dimension);
	other_help_button.setMinimumSize(button_dimension);
	other_help_button.setToolTipText("Help on other sequences");
	other_help_listener = 
	    new Help_button_listener(this,"subsection_Other_sequences");
	other_help_button.addActionListener(other_help_listener);
	add_panel(load_panel, other_help_button, load_constraints,
		  14,5,1,1,anchor_center,fill_none);

	//Output directory choose
	label_outdir = new JLabel("3.Output directory");
	label_outdir.setFont(font_text);
	add_panel(load_panel, label_outdir, load_constraints,
		  0,6,1,3,anchor_line_start,fill_none);

	outdir_text_field = new JTextField();
	outdir_text_field.setPreferredSize(textfield_dimension);
	outdir_text_field.setMinimumSize(textfield_dimension);
	add_panel(load_panel, outdir_text_field, load_constraints,
		  3,6,1,9,anchor_line_start,fill_none);


	outdir_open_button = new JButton(open_image);
	outdir_open_listener = new Open_button_listener(outdir_text_field,
						     JFileChooser.DIRECTORIES_ONLY,
						     false); 
	outdir_open_button.setPreferredSize(button_dimension);
	outdir_open_button.setMinimumSize(button_dimension);
	outdir_open_button.addActionListener(outdir_open_listener);
	outdir_open_button.setToolTipText("Select output directory");
	add_panel(load_panel, outdir_open_button, load_constraints,
		  12,6,1,1,anchor_center,fill_none);


	outdir_cancel_button = new JButton(cancel_image);
	outdir_cancel_listener = new Cancel_button_listener(outdir_text_field);
	outdir_cancel_button.setPreferredSize(button_dimension);
	outdir_cancel_button.setMinimumSize(button_dimension);
	outdir_cancel_button.addActionListener(outdir_cancel_listener);
	outdir_cancel_button.setToolTipText("Remove the actual output directory");
	add_panel(load_panel, outdir_cancel_button, load_constraints,
		  13,6,1,1,anchor_center,fill_none);


	outdir_help_button = new JButton(help_image);
	outdir_help_button.setPreferredSize(button_dimension);
	outdir_help_button.setMinimumSize(button_dimension);
	outdir_help_button.setToolTipText("Help on output directory");
	outdir_help_listener = new Help_button_listener(this,"subsection_Output");
	outdir_help_button.addActionListener(outdir_help_listener);
	add_panel(load_panel, outdir_help_button, load_constraints,
		  14,6,1,1,anchor_center,fill_none);


	//Configure sequences button
	load_constraints.insets = inset_conf_button;
	conf_seq_button = new JButton("Sequences parameters");
	conf_seq_listener = new Configure_sequences_button_listener(this);
	conf_seq_button.addActionListener(conf_seq_listener);
	conf_seq_button.setPreferredSize(conf_button_dimension);
	conf_seq_button.setMinimumSize(conf_button_dimension);
	conf_seq_button.setToolTipText("Configure other sequences");
	conf_seq_button.setFont(font_plain);
	add_panel(load_panel, conf_seq_button, load_constraints,
		  0,7,1,4,anchor_line_end,fill_none);	
	conf_seq_help_button = new JButton(help_image);
	conf_seq_help_button.setPreferredSize(button_dimension);
	conf_seq_help_button.setMinimumSize(button_dimension);
	conf_seq_help_button.setToolTipText("Help on sequences configuration");
	conf_seq_help_listener = 
	    new Help_button_listener(this,"subsection_Sequence_parameters");
	conf_seq_help_button.addActionListener(conf_seq_help_listener);
	add_panel(load_panel, conf_seq_help_button, load_constraints,
		  4,7,1,1,anchor_line_start,fill_none);


	return load_panel;

    }


    //Build Software panel
    private JPanel buildSoftPanel(){

	//Layout and constraints definitions
	soft_layout = new GridBagLayout();
	soft_constraints = new GridBagConstraints();
 	soft_constraints.weightx = 0;
  	soft_constraints.weighty = 0;
	soft_constraints.insets = inset_main;

	//Soft panel
	soft_panel = new JPanel(soft_layout);
	soft_panel.setBackground(Color.white);

	//Intro on software choose
	
	label_intro_pipeline = new JLabel("RUN ANALYSIS");
	label_intro_pipeline.setFont(font_bold);
	add_panel(soft_panel, label_intro_pipeline, soft_constraints,
		  0,0,1,22,anchor_line_start,fill_none);

	soft_constraints.insets = inset_soft;
	label_mask = new JLabel("Preprocessing");
	label_mask.setFont(font_text);
	add_panel(soft_panel, label_mask, soft_constraints,
		  0,1,1,4, anchor_center, fill_none);

	button_arrow1 = new JButton(arrow_image);
	button_arrow1.setBackground(Color.white);
	button_arrow1.setForeground(Color.blue);
	button_arrow1.setOpaque(false);
	button_arrow1.setBorderPainted(false);
	button_arrow1.setFocusPainted(false);
	button_arrow1.setContentAreaFilled(false);
	button_arrow1.setPreferredSize(button_dimension);
	button_arrow1.setMinimumSize(button_dimension);
	add_panel(soft_panel, button_arrow1, soft_constraints,
		  4,1,1,2, anchor_center, fill_none);
	
	label_alignment = new JLabel("Alignment");
	label_alignment.setFont(font_text);
	add_panel(soft_panel, label_alignment, soft_constraints,
		  6,1,1,4, anchor_center, fill_none);
	
	button_arrow2 = new JButton(arrow_image);
	button_arrow2.setBackground(Color.white);
	button_arrow2.setForeground(Color.blue);
	button_arrow2.setOpaque(false);
	button_arrow2.setBorderPainted(false);
	button_arrow2.setFocusPainted(false);
	button_arrow2.setContentAreaFilled(false);
	button_arrow2.setPreferredSize(button_dimension);
	button_arrow2.setMinimumSize(button_dimension);
	add_panel(soft_panel, button_arrow2, soft_constraints,
		  10,1,1,2, anchor_center, fill_horizontal);


	label_conserved = new JLabel("Conserved regions");
	label_conserved.setFont(font_text);
	add_panel(soft_panel, label_conserved, soft_constraints,
		  12,1,1,4, anchor_center, fill_horizontal);
	
	button_arrow3 = new JButton(arrow_image);
	button_arrow3.setBackground(Color.white);
	button_arrow3.setForeground(Color.blue);
	button_arrow3.setOpaque(false);
	button_arrow3.setBorderPainted(false);
	button_arrow3.setFocusPainted(false);
	button_arrow3.setContentAreaFilled(false);
	button_arrow3.setPreferredSize(button_dimension);
	button_arrow3.setMinimumSize(button_dimension);
	add_panel(soft_panel, button_arrow3, soft_constraints,
		  16,1,1,2, anchor_center, fill_horizontal);

	label_structure = new JLabel("RNA structure prediction");
	label_structure.setFont(font_text);
	add_panel(soft_panel, label_structure, soft_constraints,
		  18,1,1,4, anchor_center, fill_horizontal);

	//Software checkboxes
	soft_constraints.insets = inset_checkbox;
	mask_cds_checkbox = new JCheckBox("Mask known CDSs");
	mask_cds_checkbox.setBackground(Color.white);
	mask_cds_checkbox.setFont(font_plain);
	mask_cds_checkbox.setSelected(true);
	add_panel(soft_panel,mask_cds_checkbox, soft_constraints,
		  0,2,1,3,anchor_line_start,fill_none);

	button_help_mask_cds = new JButton(help_small_image);
	button_help_mask_cds.setPreferredSize(help_soft_dimension);
	button_help_mask_cds.setMinimumSize(help_soft_dimension);
	button_help_mask_cds.setToolTipText("Help on masking CDSs");
	button_help_mask_cds.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"mask_regions");
	button_help_mask_cds.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_mask_cds, soft_constraints,
		  3,2,1,1,anchor_line_start,fill_none);

	separator_vert1 = new JSeparator(JSeparator.VERTICAL);
	add_panel(soft_panel, separator_vert1, soft_constraints,
		  4,2,3,1, anchor_center,fill_vertical);
	

	yass_checkbox = new JCheckBox("YASS");
	yass_checkbox.setSelected(true);
	yass_checkbox.setFont(font_plain);
	yass_checkbox.setBackground(Color.white);
	add_panel(soft_panel,yass_checkbox, soft_constraints,
		  6,2,1,3,anchor_line_start,fill_none);
	
	button_help_yass = new JButton(help_small_image);
	button_help_yass.setPreferredSize(help_soft_dimension);
	button_help_yass.setMinimumSize(help_soft_dimension);
	button_help_yass.setToolTipText("Help on Yass software");
	button_help_yass.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"yass");
	button_help_yass.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_yass, soft_constraints,
		  9,2,1,1,anchor_line_start,fill_none);

	separator_vert2 = new JSeparator(JSeparator.VERTICAL);
	add_panel(soft_panel, separator_vert2, soft_constraints,
		  11,2,3,1, anchor_center,fill_vertical);

	button_help_cg_seq = new JButton(help_small_image);
	button_help_cg_seq.setPreferredSize(help_soft_dimension);
	button_help_cg_seq.setMinimumSize(help_soft_dimension);
	button_help_cg_seq.setToolTipText("Help on CG-seq");
	button_help_cg_seq.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"subsection_conserved_regions");
	button_help_cg_seq.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_cg_seq, soft_constraints,
		  12,2,1,4,anchor_center,fill_none);

	separator_vert3 = new JSeparator(JSeparator.VERTICAL);
	add_panel(soft_panel, separator_vert3, soft_constraints,
		  17,2,3,1, anchor_center,fill_vertical);


	carnac_checkbox = new JCheckBox("Carnac");
	carnac_checkbox.setSelected(true);
	carnac_checkbox.setFont(font_plain);
	carnac_checkbox.setBackground(Color.white);
	add_panel(soft_panel,carnac_checkbox, soft_constraints,
		  18,2,1,3,anchor_line_start,fill_none);
	
	button_help_carnac = new JButton(help_small_image);
	button_help_carnac.setPreferredSize(help_soft_dimension);
	button_help_carnac.setMinimumSize(help_soft_dimension);
	button_help_carnac.setToolTipText("Help on Carnac");
	button_help_carnac.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"carnac");
	button_help_carnac.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_carnac, soft_constraints,
		  21,2,1,1,anchor_line_start,fill_none);

	
	mask_rna_checkbox = new JCheckBox("Mask known RNAs");
	mask_rna_checkbox.setBackground(Color.white);
	mask_rna_checkbox.setFont(font_plain);
	mask_rna_checkbox.setSelected(false);
	add_panel(soft_panel,mask_rna_checkbox, soft_constraints,
		  0,3,1,3,anchor_line_start,fill_none);

	button_help_mask_rna = new JButton(help_small_image);
	button_help_mask_rna.setPreferredSize(help_soft_dimension);
	button_help_mask_rna.setMinimumSize(help_soft_dimension);
	button_help_mask_rna.setToolTipText("Help on masking RNAs");
	button_help_mask_rna.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"mask_regions");
	button_help_mask_rna.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_mask_rna, soft_constraints,
		  3,3,1,1,anchor_line_start,fill_none);


	blast_checkbox = new JCheckBox("Blast");
	blast_checkbox.setSelected(false);
	blast_checkbox.setFont(font_plain);
	blast_checkbox.setBackground(Color.white);
	add_panel(soft_panel,blast_checkbox, soft_constraints,
		  6,3,1,3,anchor_line_start,fill_none);
	
	button_help_blast = new JButton(help_small_image);
	button_help_blast.setPreferredSize(help_soft_dimension);
	button_help_blast.setMinimumSize(help_soft_dimension);
	button_help_blast.setToolTipText("Help on Blast software");
	button_help_blast.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"blast");
	button_help_blast.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_blast, soft_constraints,
		  9,3,1,1,anchor_line_start,fill_none);


	rnaz_checkbox = new JCheckBox("RNAz");
	rnaz_checkbox.setSelected(false);
	rnaz_checkbox.setBackground(Color.white);
	rnaz_checkbox.setFont(font_plain);
	add_panel(soft_panel,rnaz_checkbox, soft_constraints,
		  18,3,1,3,anchor_line_start,fill_none);
	
	button_help_rnaz = new JButton(help_small_image);
	button_help_rnaz.setPreferredSize(help_soft_dimension);
	button_help_rnaz.setMinimumSize(help_soft_dimension);
	button_help_rnaz.setToolTipText("Help on RNAz");
	button_help_rnaz.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"RNAz");
	button_help_rnaz.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_rnaz, soft_constraints,
		  21,3,1,1,anchor_line_start,fill_none);


	mask_redundancy_checkbox = new JCheckBox("Clean up redundancy");
	mask_redundancy_checkbox.setBackground(Color.white);
	mask_redundancy_checkbox.setFont(font_plain);
	mask_redundancy_checkbox.setSelected(false);
	add_panel(soft_panel,mask_redundancy_checkbox, soft_constraints,
		  0,4,1,3,anchor_line_start,fill_none);

	button_help_mask_redundancy = new JButton(help_small_image);
	button_help_mask_redundancy.setPreferredSize(help_soft_dimension);
	button_help_mask_redundancy.setMinimumSize(help_soft_dimension);
	button_help_mask_redundancy.setToolTipText("Help on cleanup redundancy");
	button_help_mask_redundancy.addMouseListener(this);	
	mask_regions_help_listener = 
	    new Help_button_listener(this,"clean_up_redundancy");
	button_help_mask_redundancy.addActionListener(mask_regions_help_listener);
	add_panel(soft_panel,button_help_mask_redundancy, soft_constraints,
		  3,4,1,1,anchor_line_start,fill_none);


	//Configure software button
	soft_constraints.insets = inset_conf_button;
	conf_soft_button = new JButton("Configure software");
	conf_soft_listener = new Configure_software_button_listener(this);
	conf_soft_button.addActionListener(conf_soft_listener);
	conf_soft_button.setPreferredSize(conf_button_dimension);
	conf_soft_button.setMinimumSize(conf_button_dimension);
	conf_soft_button.setToolTipText("Configure selected software");
	conf_soft_button.setFont(font_plain);
	add_panel(soft_panel, conf_soft_button, soft_constraints,
		  0,5,1,4,anchor_line_end,fill_none);	
	conf_soft_help_button = new JButton(help_image);
	conf_soft_help_button.setPreferredSize(button_dimension);
	conf_soft_help_button.setMinimumSize(button_dimension);
	conf_soft_help_button.setToolTipText("Help on software configuration");
	conf_soft_help_listener = 
	    new Help_button_listener(this,"section_Run_analysis");
	conf_soft_help_button.addActionListener(conf_soft_help_listener);
	add_panel(soft_panel, conf_soft_help_button, soft_constraints,
		  4,5,1,1,anchor_line_start,fill_none);

	return soft_panel;

    }
    



    //Place a component in a panel according to given
    //constraints for a GridBagLayout
    public void add_panel(JPanel panel,
			  JComponent component,
			  GridBagConstraints constraints,
			  int gridx,
			  int gridy,
			  int gridheight,
			  int gridwidth,
			  int anchor,
			  int fill){

	constraints.gridx = gridx;
	constraints.gridy = gridy;
	constraints.gridwidth = gridwidth;
	constraints.gridheight = gridheight;
	constraints.anchor = anchor;
	constraints.fill = fill;

	panel.add(component,constraints);

    }
    





    ////
    //  Component getter and setter
    ////


    //Reference text getter
    public String get_reference_text(){

	return ref_text_field.getText();

    }

    //Other directories text getter
    public String get_others_text(){

	return other_text_area.getText();

    }

    //Outdir text getter
    public String get_outdir_text(){

	return outdir_text_field.getText();

    }

    //Mask cds+rnas step wanted
    public Boolean mask_regions_wanted(){

	return (mask_cds_checkbox.isSelected() ||
		mask_rna_checkbox.isSelected());

    }

    //Mask cds wanted
    public Boolean mask_cds_wanted() {

	return mask_cds_checkbox.isSelected();
    }

    //Mask rna wanted
    public Boolean mask_rna_wanted() {

	return mask_rna_checkbox.isSelected();
    }


    //Mask redundancy step wanted
    public Boolean mask_redundancy_wanted(){

	return mask_redundancy_checkbox.isSelected();

    }

    //Yass step wanted
    public Boolean yass_wanted(){

	return yass_checkbox.isSelected();

    }

    //Blast step wanted
    public Boolean blast_wanted(){

	return blast_checkbox.isSelected();

    }

    //Find conserved region step wanted
    public Boolean conserved_wanted(){

	return true;

    }

    //Carnac step wanted
    public Boolean carnac_wanted(){

	return carnac_checkbox.isSelected();

    }

    //RNAz step wanted
    public Boolean rnaz_wanted(){

	return rnaz_checkbox.isSelected();

    }


    //Get nummber of wanted software
    public int get_software_number(){

	int nb_software = 0;
	if(mask_regions_wanted()){
	    nb_software++;
	}
	if(mask_redundancy_wanted()){
	    nb_software++;
	}
	if(yass_wanted()){
	    nb_software++;
	}
	if(blast_wanted()){
	    nb_software++;
	}
	if(conserved_wanted()){
	    nb_software++;
	}
	if(carnac_wanted()){
	    nb_software++;
	}
	if(rnaz_wanted()){
	    nb_software++;
	}


	return nb_software;

    }

    ////
    //  Configuration getter and setter
    ////

    //Get perl script location
    public String get_perl_script(){
	
	return perl_script;
    }


    //Get current masking coding configuration file
    public String get_masking_regions_conf(){
	
	return masking_regions_conf_to_use;
    }
    
    //Set current masking coding configuration file
    public void set_masking_regions_conf(String mask_conf){
	
	this.masking_regions_conf_to_use = mask_conf;
    }

    //Get default masking configuration file
    public String get_default_masking_regions_conf(){

	return default_masking_regions_conf;
    }

    //Get temporary masking configuration file
    public String get_temp_masking_regions_conf(){

	return temp_masking_regions_conf;
    }

    //Get current masking redundancy configuration file
    public String get_masking_redundancy_conf(){
	
	return masking_redundancy_conf_to_use;
    }
    
    //Set current masking redundancy configuration file
    public void set_masking_redundancy_conf(String mask_conf){
	
	this.masking_redundancy_conf_to_use = mask_conf;
    }

    //Get default masking configuration file
    public String get_default_masking_redundancy_conf(){

	return default_masking_redundancy_conf;
    }

    //Get temporary masking configuration file
    public String get_temp_masking_redundancy_conf(){

	return temp_masking_redundancy_conf;
    }


    //Get current yass configuration file
    public String get_yass_conf(){
	
	return yass_conf_to_use;
    }
    
    //Set current yass configuration file
    public void set_yass_conf(String yass_conf){
	
	this.yass_conf_to_use = yass_conf;
    }

    //Get default yass configuration file
    public String get_default_yass_conf(){

	return default_yass_conf;
    }

    //Get temporary yass configuration file
    public String get_temp_yass_conf(){

	return temp_yass_conf;
    }


    //Get current blast configuration file
    public String get_blast_conf(){
	
	return blast_conf_to_use;
    }
    
    //Set current blast configuration file
    public void set_blast_conf(String blast_conf){
	
	this.blast_conf_to_use = blast_conf;
    }

    //Get default blast configuration file
    public String get_default_blast_conf(){

	return default_blast_conf;
    }

    //Get temporary blast configuration file
    public String get_temp_blast_conf(){

	return temp_blast_conf;
    }

     //Get current cg_seq configuration file
    public String get_cg_seq_conf(){
	
	return cg_seq_conf_to_use;
    }
    
    //Set current cg_seq configuration file
    public void set_cg_seq_conf(String cg_seq_conf){
	
	this.cg_seq_conf_to_use = cg_seq_conf;
    }

    //Get default cg_seq configuration file
    public String get_default_cg_seq_conf(){

	return default_cg_seq_conf;
    }

    //Get temporary cg_seq configuration file
    public String get_temp_cg_seq_conf(){

	return temp_cg_seq_conf;
    }


    //Get current carnac configuration file
    public String get_carnac_conf(){
	
	return carnac_conf_to_use;
    }
    
    //Set current carnac configuration file
    public void set_carnac_conf(String carnac_conf){
	
	this.carnac_conf_to_use = carnac_conf;
    }

    //Get default carnac configuration file
    public String get_default_carnac_conf(){

	return default_carnac_conf;
    }


    //Get temporary carnac configuration file
    public String get_temp_carnac_conf(){

	return temp_carnac_conf;
    }


    //Get current rnaz configuration file
    public String get_rnaz_conf(){
	
	return rnaz_conf_to_use;
    }
    
    //Set current rnaz configuration file
    public void set_rnaz_conf(String rnaz_conf){
	
	this.rnaz_conf_to_use = rnaz_conf;
    }

    //Get default rnaz configuration file
    public String get_default_rnaz_conf(){

	return default_rnaz_conf;
    }

    //Get temporary rnaz configuration file
    public String get_temp_rnaz_conf(){

	return temp_rnaz_conf;
    }

    //Set sequences probabilities
    public void set_species_proba(String[] seq_proba){
	this.seq_proba = seq_proba;
    } 

    //Get sequences probabilities
    public String[] get_species_proba(){
       return seq_proba;
    } 

    //Get default probability value
    public String get_default_proba(){

	return default_proba;
    }

    //Get proba hash table
    public Hashtable<String,String> get_hash_proba(){

	return hash_proba;
    }

    //Set proba hash table
    public void set_hash_proba(Hashtable<String,String> hash_proba){

	this.hash_proba = hash_proba;
    }

    //Read the a configuration file and returns its content
    public String get_conf_content(String conf_file){

	String content = "";
	FileReader reader = null;
	BufferedReader buffer_reader = null;


	//Read the file
	try{
	    reader = new FileReader(conf_file);
	    buffer_reader = new BufferedReader(reader);
	    
	    String line = "";
	    while((line = buffer_reader.readLine()) != null){
		content += line+"\n";
	    }
	    
	}
	catch(IOException ioe){
	    ioe.printStackTrace();
	}
	finally{
	    try{
		buffer_reader.close();
		reader.close();
	    }
	    catch(IOException ioe_close){
		ioe_close.printStackTrace();
	    }
	}

	return content;


    }


    //Write new content inside the new configuration file
    public void write_conf_content(String content, String new_conf_file){

	FileWriter writer = null;
	BufferedWriter buffer_writer = null;

	try{
	    writer = new FileWriter(new_conf_file);
	    buffer_writer = new BufferedWriter(writer);	    
	    buffer_writer.write(content);
	}
	catch(IOException ioe){
	    ioe.printStackTrace();
	}
	finally{
	    try{
		buffer_writer.flush();
		buffer_writer.close();
		writer.close();
	    }
	    catch(IOException ioe_close){
		ioe_close.printStackTrace();
	    }
	}


    }


    ////
    //  Help getter
    ////

    public String get_help_file(){
	return help_file;
    }


    ////
    //  Mouse listener implementation
    ////


    //Action when mouse is clicked
    public void mouseClicked(MouseEvent e) {
    }
    
    
    //Action when mouse is pressed
    public void mousePressed(MouseEvent e) {
    }
    
    //Action when mouse if released
    public void mouseReleased(MouseEvent e) {

    }
    
    //Action when mouse entered
    public void mouseEntered(MouseEvent e) {

	//Set cursor representing hand
	setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)); 

    }
    
    public void mouseExited(MouseEvent e) {

	//Set normal cursor
	setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); 
	
    }

}
