///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;
import java.lang.InterruptedException;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.File;
import java.util.regex.Pattern;
import java.util.regex.Matcher;


//Run button listener
public class Run_button_listener implements ActionListener {


    //Main window
    private Main_window main_window;

    //Perl command
    private String perl_cmd;

    //Run window
    private Run_window run_window = null;

    //Error regular expression
    private String error_regexp;

    //Script output
    private String output;

    //Boolean that says if an error occured
    private Boolean has_error;

    //Error message content
    private String error_message;

    //Error window title
    private String error_title;

    //Error panel
    private JOptionPane error_pane;

    //Warning regular expression
    private String warning_regexp;

    //Boolean that says if a warning occured
    private Boolean has_warning;

    //Warning message content
    private String warning_message;

    //Warning window title
    private String warning_title;

    //Warning panel
    private JOptionPane warning_pane;
    
    //Content of the conserved configuration file to deal with proba
    private String content;

    //

    //Constructor
    public Run_button_listener(Main_window main_window){

	super();
	this.main_window = main_window;
	perl_cmd = "perl";
	error_regexp = "ERROR.+";
	warning_regexp = ".+warning.+";


    }


    public void actionPerformed(ActionEvent e){


	//Error panel
	error_pane = new JOptionPane();
	//Warning panel
	warning_pane = new JOptionPane();

	//Probability parameter
	String proba_parameter = "-p ";
	//Pattern and Matcher used in the probability parsing
	Pattern pattern;
	Matcher matcher;
	//Probability parameter regular expression
	String proba_regexp = "(#?-p[ \\t]+)((\\d+\\.\\d+),?)+";

	has_error = false;
	error_message = "";
	error_title = "";

	has_warning = false;
	warning_message = "";
	warning_title = "";

	//Check Files provided
	if((main_window.get_reference_text()).equals("")){
	    error_message = "You must provide a reference fasta file";
	    error_title = "ERROR : Reference file missing...";
	    has_error = true;
	}
	else{
	    if((main_window.get_others_text()).equals("")){
		error_message = "You must provide at least one other species";
		error_title = "ERROR : Other species missing...";
		has_error = true;

	    }
	    else{
		if((main_window.get_outdir_text()).equals("")){
		    error_message = 
			"You must provide an outdirecory";
		    error_title = "ERROR : Outdir missing...";
		    has_error = true;

		}

	    }
  
	}

	//Check perl script existence
	File perl_script = new File(main_window.get_perl_script());
	if(!perl_script.exists()){
	    error_message = 
		"The perl script \""+perl_script.toString()+"\" doesn't exist";
	    error_title = "ERROR : Perl script missing...";
	    has_error = true;

	}

	//An error occured
	if(has_error){
	    error_pane.showMessageDialog(main_window,
					 error_message,
					 error_title,
					 JOptionPane.ERROR_MESSAGE);

	}

	else{

	    //Check pipeline wanted
	    int nb_software = main_window.get_software_number();

	    if(nb_software != 0){


		if(main_window.conserved_wanted()){

		    //Check species probabilities
		    String[] proba = main_window.get_species_proba();

		    //No probabilities given : default one
		    if(proba == null){
			String sequences_text = main_window.get_others_text();
			int nb_sequences = (sequences_text.split(",")).length;
			proba = new String[nb_sequences];
			for(int i = 0; i < nb_sequences; i++){
			    proba[i] = main_window.get_default_proba();
			}
		    }
	

		    //Build proba parameter
		    for(int i =0; i< proba.length - 1; i++){
			proba_parameter += proba[i]+",";
		    }
		    proba_parameter += proba[proba.length - 1];

		    //Get CG-seq configuration file
		    String cg_seq_conf = main_window.get_cg_seq_conf();

		    //Get content
		    content = 
			main_window.get_conf_content(cg_seq_conf);
		    String temp_content = "";

		    //Replace parameter with new one or add it if not found
		    pattern = Pattern.compile(proba_regexp);
		    matcher = pattern.matcher(content);

		    if(matcher.find()){
			temp_content = matcher.replaceAll(proba_parameter);
		    }
		    else{
			temp_content = content+"\n\n"+proba_parameter+"\n";
		    }


		    //Save temporary cg_seq configuration file
		    main_window.write_conf_content(temp_content, 
						   main_window.get_temp_cg_seq_conf());
		
		}


		//Mask CDS+RNAs part
		if(main_window.mask_regions_wanted()){

		    String mode = "--mode ";
		    //mask CDS + RNAs mode
		    if(main_window.mask_cds_wanted() &&
		       main_window.mask_rna_wanted()){
			mode += "2";

		    }
		    else{
			//mask only CDS
			if(main_window.mask_cds_wanted()){
			    mode += "0";
			}
			//mask only RNAs
			else{
			    mode += "1";
			}

		    }

		    //Mode regular expression
		    String mode_regexp = "(#?--mode[ \\t]+)[012]";

		    //Get mask_regions configuration file
		    String mask_regions_conf = 
			main_window.get_masking_regions_conf();

		    //Get content
		    String mask_regions_content = 
			main_window.get_conf_content(mask_regions_conf);
		    String temp_content = "";

		    //Replace parameter with new one or add it if not found
		    pattern = Pattern.compile(mode_regexp);
		    matcher = pattern.matcher(mask_regions_content);

		    if(matcher.find()){
			temp_content = matcher.replaceAll(mode);
		    }
		    else{
			temp_content = mask_regions_content+"\n\n"+mode+"\n";
		    }


		    //Save temporary cg_seq configuration file
		    main_window.write_conf_content(temp_content, 
						   main_window.get_temp_masking_regions_conf());


		}


		//Construct command
		String others_text = main_window.get_others_text();
		others_text = others_text.replaceAll("[\\n\\r]+","");
		String[] others_dir = others_text.split(",");
		String []cmd;
		cmd = new String[4 + others_dir.length + nb_software];
		cmd[0] = perl_cmd;
		String perl_script_path = main_window.get_perl_script();
		//		perl_script_path = perl_script_path.replaceAll(" ","\\ ");
		cmd[1] = perl_script_path;
		cmd[2] = main_window.get_reference_text();
		for(int i = 0; i < others_dir.length; i++){
		    cmd[i+3] = others_dir[i];
		}
		cmd[others_dir.length+3] = "-o="+main_window.get_outdir_text();
		int current_software = 0;
		if(main_window.mask_regions_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--mask-regions="+main_window.get_temp_masking_regions_conf();
		}
		if(main_window.mask_redundancy_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--mask-redundancy="+
			main_window.get_masking_redundancy_conf();
		}
		if(main_window.yass_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--yass="+main_window.get_yass_conf();
		}
		if(main_window.blast_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--blast="+main_window.get_blast_conf();
		}
		if(main_window.conserved_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--conserved="+main_window.get_temp_cg_seq_conf();
		}
		if(main_window.carnac_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--carnac="+main_window.get_carnac_conf();
		}
		if(main_window.rnaz_wanted()){
		    current_software++;
		    cmd[others_dir.length+4+current_software - 1] = 
			"--rnaz="+main_window.get_rnaz_conf();
		}

		//Run command and display run window
		try{
// 		    for(String c : cmd){
// 			System.out.println(c);
// 		    }
		    Runtime runtime = Runtime.getRuntime();
		    final Process process = runtime.exec(cmd);
		    new Thread(){
			public void run(){
			    try{
				
				
				BufferedReader reader = 
				    new BufferedReader(new InputStreamReader(process.getInputStream()));
				
				

				try{
				    while((output = reader.readLine()) != null){

					//Test if an error occured
					if(output.matches(error_regexp)){
					    has_error = true;
					    error_message = output;
					    error_title = "An error occured during pipeline execution...";
					}

					//Test if a warning occured
					if(output.matches(warning_regexp)){
					    has_warning = true;
					}

				    }
				}finally{
				    reader.close();
				    
				}
			    }
			    catch(IOException ioe){
				ioe.printStackTrace();
			    }

			    if(has_error){
				run_window.setVisible(false);
				error_pane.showMessageDialog(main_window,
							     error_message,
							     error_title,
							     JOptionPane.ERROR_MESSAGE);
			    }
			    else{

				run_window.change_label();
				run_window.stop_progress_bar();
				run_window.display_results(has_warning);
			    }
			    //Rewrite the orginal cg_seq conf file
			    if(main_window.conserved_wanted()){
				main_window.write_conf_content(get_content(),
							       main_window.get_temp_cg_seq_conf());
			    }
			}

		    }.start();
		    run_window = new Run_window(main_window);
		    run_window.build();
		    

		}
		catch(IOException ioe){
		    ioe.printStackTrace();
		}
		



	    }
	    else{
		error_message = "Select at least one software...";
		error_title = "ERROR : no software selected";
		error_pane.showMessageDialog(main_window,
					     error_message,
					     error_title,
					     JOptionPane.ERROR_MESSAGE);

	    }
	}

    }

    //Get content containing conserved software configuration
    public String get_content(){
	return content;
    }


}
