///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// This file is part of CG-seq.
//
//  CG-seq is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  CG-seq is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Desktop;
import java.net.URI;
import java.net.URISyntaxException;
import java.io.IOException;
import java.io.File;

//Class that will build the run window
public class Run_window extends JDialog implements ActionListener{


    //Main panel
    private JPanel main_run_panel = null;

    //Layout manager and contraints
    private GridBagLayout gb_layout = null;
    private GridBagConstraints constraints = null;

    //Insets
    private Insets inset_space = null;

    //Progress bar
    private JProgressBar progress = null;

    //Label
    private JLabel run_label;

    //Buttons
    private JButton button_result = null;
    private JButton button_main = null;
    
    //Dimsensions
    private Dimension result_dimension = new Dimension(200,30);
    private Dimension main_dimension = new Dimension(150,30);

    //Name of the results page
    private String result_page_name = "results.html";
    private String outdir;

    public Run_window(Main_window main_window){

	super(main_window,"CG-seq message",true);
	this.setLocationRelativeTo(main_window);
	this.setResizable(false);
	this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	this.outdir = main_window.get_outdir_text();
    }



    //Build the dialog
    public void build(){


	//Insets
	inset_space = new Insets(10,10,10,10);

	//Layout and constraints definitions
	gb_layout = new GridBagLayout();
	constraints = new GridBagConstraints();
	constraints.insets = inset_space;

	//Main panel
	main_run_panel = new JPanel(gb_layout);
	main_run_panel.setBackground(Color.white);

	//Label
	run_label = new JLabel("Running... Please wait");
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 2;
	constraints.gridheight = 1;
	constraints.anchor = GridBagConstraints.CENTER;
	main_run_panel.add(run_label,constraints);
	
	//Progress Bar
	progress = new JProgressBar();
	progress.setStringPainted(false);
	progress.setIndeterminate(true);
	constraints.gridx = 0;
	constraints.gridy = 1;
	constraints.gridwidth = 2;
	constraints.gridheight = 1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	main_run_panel.add(progress,constraints);

	getContentPane().add(main_run_panel);
	pack();
	setVisible(true);

    }


    //Close the dialog
    public void close(){
	setVisible(false);
	dispose();

    }

    //Set Button enabled
    public void enable_buttons(){
	button_result.setEnabled(true);
	button_main.setEnabled(true);
    }


    //Change Label
    public void change_label(){
	run_label.setText("Computation completed");
	run_label.repaint();
    }

    //Stop Progress Bar
    public void stop_progress_bar(){
	progress.setIndeterminate(false);
	progress.setMaximum(100);
	progress.setValue(100);

    }

    //Adding buttons and result location
    public void display_results(Boolean has_warning){

	File result_file = new File(outdir);
	outdir = result_file.getAbsolutePath()+File.separator;

	JLabel label_results = 
	    new JLabel("<html>Results are available in the directory<br>\""+outdir+"\"</html>");


	constraints.gridx = 0;
	constraints.gridy = 2;
	constraints.gridwidth = 2;
	constraints.gridheight = 1;
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.LINE_START;
	main_run_panel.add(label_results,constraints);

	JLabel label_results2 = 
	    new JLabel("<html>See <br>- "+result_page_name+" for viewing your results in your favorite web browser <br>- Worksheets files (*.csv) to open the results in a spreadsheet <br> <br> - all other generated files ");

	
	constraints.gridx = 0;
	constraints.gridy = 3;
	constraints.gridwidth = 2;
	constraints.gridheight = 1;
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.LINE_START;
	main_run_panel.add(label_results2,constraints);

	int gridy = 3;

	if(has_warning){

	    JLabel label_warning = 
	    new JLabel("<html>Warning(s) occurred during execution. They can be found<br>in  file \"warnings.txt\" in the output directory");


	    gridy++;

	    constraints.gridx = 0;
	    constraints.gridy = gridy;
	    constraints.gridwidth = 2;
	    constraints.gridheight = 1;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.LINE_START;
	    main_run_panel.add(label_warning,constraints);

	}

	gridy++;

	button_result = new JButton("See results in browser");
	button_result.setPreferredSize(result_dimension);
	button_result.setMinimumSize(result_dimension);
	button_result.addActionListener(this);
	constraints.gridx = 0;
	constraints.gridy = gridy;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.CENTER;
	main_run_panel.add(button_result,constraints);

	button_main = new JButton("Close");
	button_main.setPreferredSize(main_dimension);
	button_main.setMinimumSize(main_dimension);
	button_main.addActionListener(this);
	constraints.gridx = 1;
	constraints.gridy = gridy;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	main_run_panel.add(button_main,constraints);

	getContentPane().add(main_run_panel);
	pack();


    }



    public void actionPerformed(ActionEvent e){

	//Return to main button
	if(e.getSource() == button_main){

	    close();

	}

	//Open results in browser
	else{

	    if(e.getSource() == button_result){

		//Test if desktop object is supported
		if(Desktop.isDesktopSupported()){

		    Desktop desktop = Desktop.getDesktop();

		    //Test is browser action is supported
		    if(desktop.isSupported(Desktop.Action.BROWSE)) {

			try{

			    File result_file = new File(outdir);
			    outdir = result_file.getAbsolutePath()+"/";

			    outdir = outdir.replaceAll("\\\\","/");
			    outdir = outdir.replaceAll(" ","%20");

			    URI result_page = 
				new URI("file:///"+outdir+result_page_name);

			    File test_file = new File(result_page);

			    if(test_file.exists()){
				try{
				    desktop.browse(result_page);
				}
				catch(IOException ioe){		    
				    look_for_browser(result_page);
				}
			    }

			    else{

				result_file_error(test_file.toString());

			    }

			}
			catch(IOException ioe){
			    ioe.printStackTrace();
			}
			catch(URISyntaxException urie){
			    urie.printStackTrace();
			}

		    }
		    else{
			browser_error();
		    }

		}
		else{
		    browser_error();
		}

	    }

	}


    }


    //Show an error panel when the browser cannot be launched to
    //display results
    public void browser_error(){

	JOptionPane error_pane = new JOptionPane();

	String error_message = "The browser cannot be launched...";

	String error_title = "ERROR : Cannot launch the browser";

	error_pane.showMessageDialog(this,
				     error_message,
				     error_title,
				     JOptionPane.ERROR_MESSAGE);


    }



    //Show an error panel when the result file doesn't exist
    public void result_file_error(String result_file){

	JOptionPane error_pane = new JOptionPane();

	String error_message = 
	    "The result file "+result_file+" doesn't exist...";

	String error_title = "ERROR : Cannot find result file";

	error_pane.showMessageDialog(this,
				     error_message,
				     error_title,
				     JOptionPane.ERROR_MESSAGE);


    }


    private static void look_for_browser(URI uri) throws IOException {
	String[] commands = {
	    System.getenv("BROWSER"),
	    "xdg-open",
	    "kfmclient exec",
	    "exo-open",	
	    "gnome-open",
	    "firefox",
	    "konqueror",
	    "netscape"
	};
	
	for (String cmd : commands) {
	    if (cmd !=null) {
		try {
		    browse(cmd, uri);
		    return; 
		} catch (IOException e) {
		    System.err.println(cmd + " : " + e.getMessage());
		}
	    }
	}
	throw new IOException("no browser found");	
    }
    
    private static void browse(String command, URI uri) throws IOException {
// 	String[] args = command.split(" +");
// 	args = Arrays.copyOf(args, args.length+1);
// 	args[args.length-1] = uri.toString();
	
	String args = command + " " + uri.toString();
	final Runtime runtime = Runtime.getRuntime();
	final Process process = runtime.exec(args);
	process.getOutputStream().close();
	process.getInputStream().close();
	process.getErrorStream().close();
    }

}
