/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file aln_matrices.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that contains functions for the mangement of matrices
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define functions for the mangement of matrices
 *
 *
 */


/*Intern inclusion*/
#include "aln_matrices.h"


/******************************************************************************
 **   Definition of a basic blosum DNA matrix
 **     
 *****************************************************************************/

blosum_dna_matrix_t dna_full = {
  {5,-4,-4,-4, -2},
  {-4,5,-4,-4, -2},
  {-4,-4,5,-4, -2},
  {-4,-4,-4,5, -2},
  {-2,-2,-2,-2,-1}};


/******************************************************************************
 **   Definition of IUPAC blosum DNA matrices
 **     
 *****************************************************************************/

blosum_iupac_matrix_t iupac_mat = {
  {5,-4,-4,-4,-4,1,1,-4,-4,1,-4,-1,-1,-1,-2,-4},
  {-4,5,-4,-4,-4,1,-4,1,1,-4,-1,-4,-1,-1,-2,5},
  {-4,-4,5,-4,1,-4,1,-4,1,-4,-1,-1,-4,-1,-2,-4},
  {-4,-4,-4,5,1,-4,-4,1,-4,1,-1,-1,-1,-4,-2,-4},
  {-4,-4,1,1,-1,-4,-2,-2,-2,-2,-1,-1,-3,-3,-1,-4},
  {1,1,-4,-4,-4,-1,-2,-2,-2,-2,-3,-3,-1,-1,-1,1},
  {1,-4,1,-4,-2,-2,-1,-4,-2,-2,-3,-1,-3,-1,-1,-4},
  {-4,1,-4,1,-2,-2,-4,-1,-2,-2,-1,-3,-1,-3,-1,1},
  {-4,1,1,-4,-2,-2,-2,-2,-1,-4,-1,-3,-3,-1,-1,1},
  {1,-4,-4,1,-2,-2,-2,-2,-4,-1,-3,-1,-1,-3,-1,-4},
  {-4,-1,-1,-1,-1,-3,-3,-1,-1,-3,-1,-2,-2,-2,-1,-1},
  {-1,-4,-1,-1,-1,-3,-1,-3,-3,-1,-2,-1,-2,-2,-1,-4},
  {-1,-1,-4,-1,-3,-1,-3,-1,-3,-1,-2,-2,-1,-2,-1,-1},
  {-1,-1,-1,-4,-3,-1,-1,-3,-1,-3,-2,-2,-2,-1,-1,-1},
  {-2,-2,-2,-2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-2},
  {-4,5,-4,-4,-4,1,-4,1,1,-4,-1,-4,-1,-1,-2,5}};


blosum_iupac_matrix_t iupac_full = {
  {5,-4,-4,-4,-4,1,1,-4,-4,1,-4,-1,-1,-1,-2,-4},
  {-4,5,-4,-4,-4,1,-4,1,1,-4,-1,-4,-1,-1,-2,5},
  {-4,-4,5,-4,1,-4,1,-4,1,-4,-1,-1,-4,-1,-2,-4},
  {-4,-4,-4,5,1,-4,-4,1,-4,1,-1,-1,-1,-4,-2,-4},
  {-4,-4,1,1,-1,-4,-2,-2,-2,-2,-1,-1,-3,-3,-1,-4},
  {1,1,-4,-4,-4,-1,-2,-2,-2,-2,-3,-3,-1,-1,-1,1},
  {1,-4,1,-4,-2,-2,-1,-4,-2,-2,-3,-1,-3,-1,-1,-4},
  {-4,1,-4,1,-2,-2,-4,-1,-2,-2,-1,-3,-1,-3,-1,1},
  {-4,1,1,-4,-2,-2,-2,-2,-1,-4,-1,-3,-3,-1,-1,1},
  {1,-4,-4,1,-2,-2,-2,-2,-4,-1,-3,-1,-1,-3,-1,-4},
  {-4,-1,-1,-1,-1,-3,-3,-1,-1,-3,-1,-2,-2,-2,-1,-1},
  {-1,-4,-1,-1,-1,-3,-1,-3,-3,-1,-2,-1,-2,-2,-1,-4},
  {-1,-1,-4,-1,-3,-1,-3,-1,-3,-1,-2,-2,-1,-2,-1,-1},
  {-1,-1,-1,-4,-3,-1,-1,-3,-1,-3,-2,-2,-2,-1,-1,-1},
  {-2,-2,-2,-2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-2},
  {-4,5,-4,-4,-4,1,-4,1,1,-4,-1,-4,-1,-1,-2,5}};
