/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *  
 * \file blast_lexer.l
 *  
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-bolet@inria.fr>
 *
 * \brief Lexer file that defines the lexical analysor
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all lexical terms to recognized a blast tabular
 * format
 *
 *
 *
 */


%{
#include "blast_interface.h"
#include "blast_parser.h"
#include "macros.h"
#include <string.h>
#include <assert.h>

%}

%option yylineno
%option never-interactive


TAB         [\t]
ENDLINE     [\n]
COMA        \.
SPACE       [[:blank:]]*

INT         [0-9]+
POWER       [eE][+-]?{INT}
REAL        {INT}({COMA}{INT})?{POWER}?

NAME        [-=+_\(\)\[\]`'\"\*a-zA-Z0-9{TAB}:\.\|,/;]+

COMMENT     ^#.*



%%
{INT}  {
  blastlval.number = atoi(blasttext);
  return INT_VALUE;

}

{REAL} {
  blastlval.real = atof(blasttext);
  return REAL_VALUE;
}


{NAME} {
  NEW(blastlval.text, char, (1 + blastleng));
  strncpy(blastlval.text, blasttext, blastleng);
  blastlval.text[blastleng] = '\0';
  return NAME_VALUE;
}

{TAB}  { }

{SPACE} { }

{ENDLINE} { }

{COMMENT} { }

.  { 
  blasterror("invalid token"); 
}      


%%

int blastwrap(){
  return 1;
}


void free_flex_mem_blast() {
  /* Pop the buffer stack, destroying each element. */
  while (YY_CURRENT_BUFFER){
    yy_delete_buffer (YY_CURRENT_BUFFER );
    YY_CURRENT_BUFFER_LVALUE = NULL;
    yypop_buffer_state ();
  }
    
  /* Destroy the stack itself. */
  yyfree ( yy_buffer_stack );
  yy_buffer_stack = NULL;
    
}  /* For non-reentrant C scanner only. */