/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file clique.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines all functions used in the clique search
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all functions used in the naive and maxiclique
 * search
 *
 *
 */


#ifndef __CLIQUE_H__
#define __CLIQUE_H__


/*Intern inclusions*/
#include "types.h"


/**
 *
 * \fn void search_naive_clique(p_main_structure m)
 *
 * \brief Function used to search the naive clique for all sets
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will search the naive clique for all sets. It will include
 * by reference identity order each sequences and abort as soon as one
 * cannot be included
 *
 */


extern void search_naive_clique(p_main_structure m);


/**
 *
 * \fn void recursive_search_maxi_clique(p_set_of_seqs set,
 *                                       p_clique temp_clique,
 *                                       int index)
 *
 * \brief Function used to search recursively the maxi clique in the
 * current set
 *
 * \param[in,out] set : the current set
 * \param[in,out] temp_clique : the temporary maxi clique
 * \param[in] index : the index in the query array of the sequence to add 
 *
 * That function will search the maxi clique recursively by trying to 
 * add the query sequence in the temporary maxi clique
 *
 */

extern void recursive_search_maxi_clique(p_set_of_seqs set,
					 p_clique temp_clique,
					 int index);


/**
 *
 * \fn void maxi_clique_search(p_main_structure m)
 *
 * \brief Function used to search the maxi clique for all sets
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will search the maxi clique search for all sets. If a
 * number of sequences has been provided by the user the maxi clique will
 * be at least that size
 *
 */

extern void maxi_clique_search(p_main_structure m);


#endif
