%{
#ifdef HAVE_CONFIG_H
    #include "config.h"
#endif

#include "carnac_interface.h"
#include "carnac_parser.h"
#include "opts.h"
#include "display.h"
#include <string.h>
#include <assert.h>
#include <ctype.h>
%}


%option prefix="arnica"
%option outfile="carnac_lexer.c"


%option yylineno
%option nounput
%option never-interactive


SPACE           [[:blank:]]
SPACES          ({SPACE}+)
CRLF            (([\r][\n])|[\r]|[\n])
CRLFN           ({CRLF}+)

FASTA_GDE_TITLE     ([>#][^\r\n]+{CRLF})
CLUSTAL_TITLE   (CLUSTAL[^\r\n]*{CRLF})

NUMBER_ELEMENT   ([[:digit:]]+)

SEQUENCE_ELEMENT ([[:alpha:]\*\.\-]{1,10000})
CLUSTAL_SEQ_NAME ([[:alpha:][:digit:]_\~\#\-\.=+()\|/]+)
CLUSTAL_SEP_ELMT ([ \t\*.:\|]*{CRLF})


%x FASTA_GDE CLUSTAL_NAME CLUSTAL_SEQ CLUSTAL_NAME_SEP


%%

<INITIAL,FASTA_GDE>{FASTA_GDE_TITLE} {
  int nbc, i;
  i = 0;
  while(arnicatext[i] == ' ' || arnicatext[i] == '>') i++;
  nbc = arnicaleng-1;
  while((arnicatext[nbc] == '\n') || (arnicatext[nbc] == '\r') || (arnicatext[nbc] == ' ')) nbc--;
  NEW(arnicalval.text, char, (1+nbc-i+1));
  strncpy(arnicalval.text, &(arnicatext[i]), (1+nbc-i+1));
  arnicalval.text[nbc+1-i] = '\0';
  BEGIN(FASTA_GDE);
  if(arnicalval.text[0] == '>') {
    return FASTA_TITLE_T;
  }
  return GDE_TITLE_T;
}

<FASTA_GDE,CLUSTAL_SEQ>{SEQUENCE_ELEMENT} {
  int ii;

  NEW(arnicalval.text, char, (1 + arnicaleng));

  for(ii = 0; ii < arnicaleng; ii++) {
	 arnicalval.text[ii] = toupper(arnicatext[ii]);
  }
  arnicalval.text[arnicaleng] = '\0';

  return SEQUENCE_ELEMENT_T;
}

<FASTA_GDE,CLUSTAL_SEQ>{NUMBER_ELEMENT} {}

<INITIAL,FASTA_GDE,CLUSTAL_NAME>{CRLFN} {}
<INITIAL,FASTA_GDE,CLUSTAL_SEQ>{SPACES} {}


<INITIAL>{CLUSTAL_TITLE} {
  BEGIN(CLUSTAL_NAME);
}


<CLUSTAL_NAME,CLUSTAL_NAME_SEP>{CLUSTAL_SEQ_NAME} {
  int nbc, i;
  i = 0;
  while(arnicatext[i] == ' ') i++;
  nbc = arnicaleng-1;
  while((arnicatext[nbc] == '\n') || (arnicatext[nbc] == '\r') || (arnicatext[nbc] == ' ')) nbc--;
  NEW(arnicalval.text, char, (1+nbc-i+1));
  strncpy(arnicalval.text, &(arnicatext[i]), (1+nbc-i+1));
  arnicalval.text[nbc+1-i] = '\0';
  BEGIN(CLUSTAL_SEQ);
  return CLUSTAL_SEQ_NAME_T;
}

<CLUSTAL_NAME_SEP>{CLUSTAL_SEP_ELMT} {
  BEGIN(CLUSTAL_NAME);
  return CLUSTAL_SEP_ELMT_T;
}

<CLUSTAL_SEQ>{CRLF} { BEGIN(CLUSTAL_NAME_SEP); }

<*>. { arnicaerror("invalid token"); }

%%

int arnicawrap() {
  BEGIN(INITIAL);
  return 1;
}


