%{
#ifdef HAVE_CONFIG_H
    #include "config.h"
#endif

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
   
#include "display.h"
#include "opts.h"
#include "carnac_interface.h"
#include "alignment.h"


   char *arnica_file_name;
   int arnica_nb_errors;
   sequence* arnica_result;

   %}


%file-prefix="arnica"
%name-prefix="arnica"
%defines
%token-table
%output="carnac_parser.c"





%union {
   char car;
   sequence* sequences;
   sequence sequence;
   char *text;
   arnica_buffer buffer;
}


%token <text> FASTA_TITLE_T
%token <text> GDE_TITLE_T
%token <text> SEQUENCE_ELEMENT_T
%token <text> CLUSTAL_SEQ_NAME_T
%token CLUSTAL_SEP_ELMT_T


%type <buffer> sequence

%type <sequences> sequences multifasta multigde clustal_seqs clustal clustal_blocks clustal_block
%type <sequence> fasta gde clustal_seq


%start sequences

%%

sequences:
multifasta
{ arnica_result = $1; }
|
multigde
{ arnica_result = $1; }
|
clustal
{ arnica_result = $1; }
;


multifasta:
multifasta
fasta
{
   sequence* res = $1;
   int i = 0;
   
   while(res[i]) i++;
   
   RENEW(res,sequence, (i + 2));
   res[i] = $2;
   res[i+1] = NULL;
   $$ = res;
}
|
fasta
{
   sequence* res;

   NEW(res,sequence,2);

   res[0] = $1;
   res[1] = NULL;
   $$ = res;
}
;

multigde:
multigde
gde
{
   sequence* res = $1;
   int i = 0;
   
   while(res[i]) i++;
   
   RENEW(res,sequence, (i + 2));
   res[i] = $2;
   res[i+1] = NULL;
   $$ = res;
}
|
gde
{
   sequence* res;

   NEW(res,sequence,2);

   res[0] = $1;
   res[1] = NULL;
   $$ = res;
}
;

fasta:
FASTA_TITLE_T
sequence
{
   arnica_buffer seq = $2;
   sequence res;

   BUFFER_RELEASE(seq);

   res = init_sequence(arnica_file_name, $1, $1, BUFFER_TOSTRING(seq), BUFFER_SIZE(seq), 1);

   DESTROY(($1));

   DESTROY_BUFFER(seq);

   $$ = res;
}
;

gde:
GDE_TITLE_T
sequence
{
   arnica_buffer seq = $2;
   sequence res;

   BUFFER_RELEASE(seq);

   res = init_sequence(arnica_file_name, $1, $1, BUFFER_TOSTRING(seq), BUFFER_SIZE(seq), 1);

   DESTROY(($1));

   DESTROY_BUFFER(seq);

   $$ = res;
}
;


sequence:
sequence
SEQUENCE_ELEMENT_T
{
   arnica_buffer res = $1;
   register int i;
   int length;
   char *txt = $2;

   length = strlen(txt);

   if(BUFFER_MAXSIZE(res) < (BUFFER_SIZE(res) + length)) {
      BUFFER_RESIZE(res, (BUFFER_SIZE(res) + length + INPUT_BUFFER));
   }

   for(i = 0; i < length; i++) {
      BUFFER_PUT(res, txt[i]);
   }

   DESTROY(txt);

   $$ = res;
}
|
{
   arnica_buffer res;

   NEW_BUFFER(res, INPUT_BUFFER);

   $$ = res;
}
;









clustal:
clustal_blocks
{ $$ = $1; }
;


clustal_blocks:
clustal_blocks
clustal_block
{
   sequence *resa = $1;
   sequence *resb = $2;

   int i,j;
   
   i = 0;
   while(resa[i]) i++;

   j = 0;
   while(resb[j]) j++;

   if(i != j) {
      arnicaerror("invalid number of sequences in CLUSTALW alignment...");
   } else {

      i = 0;
      while((i < j) && (strcmp(resa[i]->name, resb[i]->name) == 0)) {
	 concat_sequence(resa[i], resb[i]);
 	 DESTROY(resb[i]->file);
 	 DESTROY(resb[i]->name);
 	 DESTROY(resb[i]->input_name);
 	 DESTROY(resb[i]->bases);
 	 DESTROY(resb[i]);
	 i++;
      }
      
      if(i < j) {
	 arnicaerror("invalid sequence names in CLUSTALW alignment...");
      }

      DESTROY(resb);
   }

   $$ = resa;
}
|
clustal_block
{ $$ = $1; }
;


clustal_block:
clustal_seqs
CLUSTAL_SEP_ELMT_T
{ $$ = $1; }
;


clustal_seqs:
clustal_seqs
clustal_seq
{
   sequence* res = $1;
   int i = 0;
   
   while(res[i]) i++;
   
   RENEW(res,sequence, (i + 2));
   res[i] = $2;
   res[i+1] = NULL;
   $$ = res;
}
|
clustal_seq
{
   sequence *res = NULL;

   NEW(res,sequence,2);

   res[0] = $1;
   res[1] = NULL;
   $$ = res;
}
;

clustal_seq:
CLUSTAL_SEQ_NAME_T
SEQUENCE_ELEMENT_T
{
   sequence res = NULL;
   char* name = $1;
   char* seq = $2;

   res = init_sequence(arnica_file_name, name, name, seq, strlen(seq), 1);

   DESTROY(name);
   DESTROY(seq);
   
   $$ = res;
}
;





%%


   void
      arnicawarning_token(const char *s, const char *token) {
      WARN_ "'%s' line '%i' token '%s': %s\n", arnica_file_name, arnicalineno, token, s _WARN;
   }
   
   void
      arnicaerror_token(const char *s, const char *token) {
      arnica_nb_errors++;
      ERROR_ "'%s' line '%i' token '%s': %s\n", arnica_file_name, arnicalineno, token, s _ERROR;
   }
   
