/*
 *  YASS 1.12
 *  Copyright (C) 2004-2007
 *  YASS Team
 *  Laurent Noe, Gregory Kucherov, Mikhail Roytberg, 
 *  Steven Corroy, Christophe Valmir.
 *
 *  laurent.noe|<A>|lifl.fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the CeCILL License as published by
 *  the CEA-CNRS-INRIA; either version 2 of the License, or (at your
 *  option) any later version, and the GNU General Public License as
 *  published by the Free Software Foundation; either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This software contains code derived from the GNU libavl library.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <stdio.h>
#include <stdlib.h>

/* 1) include utils macro */
#include "util.h"
/* 2) include global variables */
#include "global_var.h"
/* 3) the current file defs */
#include "align.h"
/* 4) other files */
#include "kword.h"
#include "tuple.h"
#include "prdyn.h"
#include "display.h"
#include "util.h"


#define FLAGSAVE() {                                \
 flagsave = (total_score >= MAX(0,mem_total_score));\
                                                    \
}

#define FLAGDROP() {                                        \
  flagdrop = (total_score  + gp_xdrop <  mem_total_score ); \
                                                            \
                                                            \
}


#define IF_FLAGDROP_MEMORISE_TMP_IN_DISPLAY_AND_RESET(){    \
 FLAGDROP();                                                \
 if ( flagdrop ){                                           \
    MEMORISE_TMP_IN_DISPLAY_AND_RESET();                    \
 }                                                          \
}



#define IF_FLAGSAVE_MEMORISE_IN_TMP()  \
{                                      \
  FLAGSAVE();                          \
  if ( flagsave ) {                    \
     MEMORISE_IN_TMP();                \
  }                                    \
}


#define MEMORISE_IN_TMP() {                              \
  mem_first_tuple_aligned = first_tuple_aligned;         \
  mem_last_tuple_aligned  = last_tuple_aligned;          \
  mem_total_score         = total_score;                 \
}


/*
 *
 */


#define MEMORISE_TMP_IN_DISPLAY_AND_RESET()  {                                   \
 if (mem_first_tuple_aligned != NULL) {                                          \
   int left_score  = 0,                                                          \
       left_pos_begin  = TBL_POS(mem_first_tuple_aligned),                       \
       right_pos_begin = TBR_POS(mem_first_tuple_aligned),                       \
       right_score     = 0,                                                      \
       left_pos_end    = TEL_POS(mem_last_tuple_aligned),                        \
       right_pos_end   = TER_POS(mem_last_tuple_aligned) ;                       \
   int tbl_space   = MIN(left_pos_begin,gp_rho_stat);                            \
   int tbr_space   = MIN(right_pos_begin,gp_rho_stat);                           \
   int tel_space   = MIN(MAX(datasize_query -  left_pos_end,0),gp_rho_stat);     \
   int ter_space   = MIN(MAX(datasize_text  - right_pos_end,0),gp_rho_stat);     \
   double entropy_query,entropy_text;                                            \
   if( tbl_space == gp_rho_stat && tbr_space == gp_rho_stat)                     \
     left_alignment_SG_Lz(                                                       \
        data_query, TBL_POS(mem_first_tuple_aligned),                            \
        tbl_space,                                                               \
        data_text,  TBR_POS(mem_first_tuple_aligned),                            \
        tbr_space,                                                               \
        gp_xdrop, &left_score, &left_pos_begin, &right_pos_begin, feature);      \
   else                                                                          \
     left_alignment_SG(                                                          \
        data_query, TBL_POS(mem_first_tuple_aligned),                            \
        tbl_space,                                                               \
        data_text,  TBR_POS(mem_first_tuple_aligned),                            \
        tbr_space,                                                               \
        gp_xdrop, &left_score, &left_pos_begin, &right_pos_begin, feature);      \
                                                                                 \
    mem_total_score += left_score;                                               \
                                                                                 \
   if( tel_space == gp_rho_stat && ter_space == gp_rho_stat)                     \
     right_alignment_SG_Lz(                                                      \
       data_query, TEL_POS(mem_last_tuple_aligned),                              \
       tel_space,                                                                \
       data_text,  TER_POS(mem_last_tuple_aligned),                              \
       ter_space,                                                                \
       gp_xdrop, &right_score, &left_pos_end, &right_pos_end,feature);           \
   else                                                                          \
     right_alignment_SG(                                                         \
       data_query, TEL_POS(mem_last_tuple_aligned),                              \
       tel_space,                                                                \
       data_text,  TER_POS(mem_last_tuple_aligned),                              \
       ter_space,                                                                \
       gp_xdrop, &right_score, &left_pos_end, &right_pos_end,feature);           \
                                                                                 \
     mem_total_score += right_score;                                             \
                                                                                 \
     MinScoreOnCountMA(feature,mem_total_score);                                 \
     entropy_query = Entropy(data_query, left_pos_begin,                         \
                             left_pos_end-left_pos_begin+1,3);                   \
     entropy_text  = Entropy(data_text,  right_pos_begin,                        \
                             right_pos_end-right_pos_begin+1,3);                 \
      if (entropy_query > gp_entropy_min &&                                      \
          entropy_text  > gp_entropy_min &&                                      \
          mem_total_score >= feature->MAminscore/2 ) {                           \
          CreateMA(left_pos_begin,                                               \
                   right_pos_begin,                                              \
                   left_pos_end,                                                 \
                   right_pos_end,                                                \
                   tuple_list,                                                   \
                   mem_first_tuple_aligned,                                      \
                   mem_last_tuple_aligned,                                       \
                   mem_total_score,                                              \
                   MIN(entropy_query,entropy_text),                              \
                   feature);                                                     \
     }                                                                           \
                                                                                 \
     mem_first_tuple_aligned = NULL;                                             \
 }                                                                               \
 /* Reset all to 0  */                                                           \
 mem_total_score = 0 ;                                                           \
 total_score =  seedscore;                                                       \
 if (t){                                                                         \
    first_tuple_aligned = t;                                                     \
    last_tuple_aligned  = t;                                                     \
 } else {                                                                        \
    first_tuple_aligned = NULL;                                                  \
    last_tuple_aligned  = NULL;                                                  \
 }                                                                               \
 IF_FLAGSAVE_MEMORISE_IN_TMP();                                                  \
}



/***********************************************************/



#ifdef INLINE
inline
#endif
int AlignTuples( tuplelist * tuple_list,
                 char * data_query, int datasize_query,
                 char * data_text,  int datasize_text,
                 Feature *feature)
{
  int left_gap,right_gap;
  int score = 0, seedscore = 0;
  int flagsave = 0, flagdrop = 0;
  int left_correction = feature->left_correction;

  /* first and last tuples in the currrent alignment */
  tuple * first_tuple_aligned =  tuple_list->first_tuple;
  tuple * last_tuple_aligned  =  tuple_list->first_tuple;
  tuple * t                   =  tuple_list->first_tuple->next;

#ifdef PREFETCH
  __builtin_prefetch(&data_query[TBL_POS(tuple_list->first_tuple)],0);
  __builtin_prefetch(&data_text[TBR_POS(tuple_list->first_tuple)],0);
  __builtin_prefetch(t,0);
#endif

  int total_score =  alignment_SG_Strait(data_query, TBL_POS(tuple_list->first_tuple),
                                         data_text , TBR_POS(tuple_list->first_tuple),
                                         TSIZE(tuple_list->first_tuple),
                                         feature
                                         );

#ifdef DEBUG_ALIGN
        DisplayTupleList(tuple_list);
        fprintf(stdout,"*** first seedscore : %d \n", total_score);
#endif
  /* temporary mem vars */
  tuple * mem_first_tuple_aligned = NULL;
  tuple * mem_last_tuple_aligned  = NULL;
  int mem_total_score = 0;

  MEMORISE_IN_TMP();

  /* one chain alignment */
  while(t)
    {
#ifdef PREFETCH
      __builtin_prefetch(&data_query[TBL_POS(t)],0);
      __builtin_prefetch(&data_text[TBR_POS(t)],0);
      __builtin_prefetch(t->next,0);
#endif

    ta:

      left_gap    =  TGAP_L( last_tuple_aligned, t);
      right_gap   =  TGAP_R( last_tuple_aligned, t);

      if ( left_gap >= 0 &&  right_gap  >= 0 ){

        /* compute sub alignments scores */

        seedscore =  alignment_SG_Strait(data_query, TBL_POS(t),
                                         data_text , TBR_POS(t),
                                         TSIZE(t),
                                         feature);
#ifdef DEBUG_ALIGN
        fprintf(stdout,"*** seedscore : %d \n", seedscore);
#endif

        score =  alignment_SG_Border (
                                      data_query, TEL_POS(last_tuple_aligned),  left_gap,
                                      data_text , TER_POS(last_tuple_aligned), right_gap,
                                      gp_xdrop+seedscore,
                                      feature);

#ifdef DEBUG_ALIGN
        fprintf(stdout,"*** score : %d \n", score);
#endif

        total_score += score + seedscore;
        last_tuple_aligned  = t;

        IF_FLAGSAVE_MEMORISE_IN_TMP();
        IF_FLAGDROP_MEMORISE_TMP_IN_DISPLAY_AND_RESET();

        t = t ->next;

      }else{
        /* some gaps are negatives ?? lead to alignments which are not coherents ...  */
        /* (1) next tuple does not extend any alignment ... removed ... */
        if (TEL_POS(t) < TEL_POS(last_tuple_aligned) || TER_POS(t) < TER_POS(last_tuple_aligned)){
          last_tuple_aligned->next = t->next;
          FREE(t,sizeof(tuple));
          t = last_tuple_aligned->next;
        }else{
          /* (2.1) next tuple increse the next alignment on both sequences but overlaps with previous part found */
          if ((t->leftsize) > 0){
            (t->leftsize)--;
            if (t->leftsize<0) {
              _WARNING("negative tuple size inside AlignTuples");
            }

            goto ta;
          }
          /* (2.2) problem with the overlap which cannot be solved in an efficient way so we remove this tuple ... */
          last_tuple_aligned->next = t->next;
          FREE(t,sizeof(tuple));
          t = last_tuple_aligned->next;
        }
      }/*if ( left_gap >= 0 &&  right_gap >= 0)       */
    }/*while(t)  */
  /*  memorise possibly tmp into displayed elements */
  MEMORISE_TMP_IN_DISPLAY_AND_RESET();
  return 0;
}

int AlignAndFree(
                 char * data_query, int datasize_query,
                 char * data_text,  int datasize_text,
                 int force,
                 Feature *feature) {

  /* list of lists manipulating vars */
  tuplelist * tl_ = NULL , *tl_prev_ = NULL, *tl_last_;
  tuple     * t_  = NULL;
  int       countreturn = 5;
  /* statistical vars */
  int number_of_tuples = 0;


  /* head lists selection and processing (alignement) :
   *
   *  Those lists are the older ones, so if i_list < gv_i_current - threshold,
   *  those lists will not be used by assemble
   *  we can process those and free some memory space
   */


  /*[a] tuplelists start  */
  volatile int i_current  = feature->i_current;
  tl_prev_ = tl_ = feature->first_tl;
  tl_last_ =       feature->last_tl;

  while ((force && (tl_ != NULL)) || (!force && (tl_ != tl_last_))){

#ifdef PREFETCH
    __builtin_prefetch(tl_->next,0);
#endif


    t_ = tl_->first_tuple;
    number_of_tuples = 0;

    /* (1) list used by ASSEMBLE and ALIGN (beware of reading/writing) */
    while (t_ != NULL){

#ifdef PREFETCH
      __builtin_prefetch(t_->next,0);
#endif

      /* if list goes further, we do not consider it
       * and we stop this algorithm :
       * - all the following list have also chances to
       *   have their last element inside consideration
       *   zone
       */

      if (!force)
        if ( t_->occurrence >= i_current - gp_rho_stat - gp_delta_stat - gp_seeds_span_max){
          countreturn--;
          if (!countreturn)
            return 0;
          else
            goto nexttuplelist;
        }

      number_of_tuples++;
      t_ = t_->next;
    }/* (1) */

    /* (2) specific consideration of each tuple list */

    /* lists iteration has not given any element in the active update
     * zone. Those alignements have been "completed" for the assemble
     * algorithm and thus can be aligned
     */

    if (number_of_tuples <= 0){

      /* (2.0) The first list is always empty (do nothing
       * and jump to the next list)
       */
    nexttuplelist:
      tl_prev_ = tl_ ;
      tl_      = tl_->next;
    }else{



      /* (2.1) alignements of one of several tuples
       */

      STATS_NB_CHAINS_TESTED_INC(feature);

      AlignTuples(tl_,
                  data_query, datasize_query,
                  data_text , datasize_text,
                  feature);

      if (!force)
        FreeTupleList(&tl_,&tl_prev_,&(tl_last_)); /* however tl_last_ must never be reached */
      else
        FreeTupleList(&tl_,&tl_prev_,&(feature->last_tl));
    }
  }/* [a] tuple list iteration */

  return 0;
}




/*
 *   Function to be lauched by threads
 */

#ifdef THREAD_ASSEMBLE_ALIGN
#ifdef WIN32
DWORD WINAPI thread_work_align(PVOID fvoid)
#else
     void * thread_work_align(void * fvoid)
#endif

#else
     void * thread_work_align(void * fvoid)
#endif
{
  Feature * f = (Feature *) fvoid;

  /*
   * [] Align work done here
   */

  AlignAndFree(
               f->chunk_query,
               gp_querysize_chunk,
               gp_text + gp_chunkstrt_text[f->i_chunk],
               gp_chunksize_text[f->i_chunk],
               0,
               f);

#ifdef THREAD_ASSEMBLE_ALIGN
  END_THREAD();
#else
  return 0;
#endif
}
